<?php
/**
* phpCan v.1
*
* $Id: class_session.php 197 2009-06-29 18:44:58Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Session {
	private $installed;
	private $logged;
	public $user;

	public function __construct () {
		global $config, $data, $debug;

		session_name('phpcan_admin');
		session_start();

		$data->connect();
		$this->installed = $data->selectCount($data->phpcan_tables['admin'], array(
			'conditions' => array(
				'name' => 'password',
				'type' => 'preferences'
			)
		));

		if ($this->installed === 0) {
			$this->logged = true;

			if (!strstr(getenv('REQUEST_URI'), 'update-db')) {
				redirect(path('update-db'));
			}
		} else {
			$this->logged = $this->check($_SESSION['user'], $_SESSION['pass']);
		}
	}

	public function isInstalled () {
		return $this->installed?true:false;
	}

	public function logged () {
		return $this->logged;
	}

	public function login () {
		global $data, $vars;

		$user = $vars->str('user');
		$password = md5($vars->str('password'));

		if ($this->check($user, $password)) {
			$_SESSION['user'] = $user;
			$_SESSION['pass'] = $password;

			session_write_close();

			return true;
		}

		return false;
	}

	public function logout () {
		session_unset();
		session_destroy();
	}

	private function check ($user, $password) {
		global $config, $data, $debug;

		$table_admin = $data->phpcan_tables['admin'];

		$userdata = $data->select($table_admin, array(
			'fields' => '*',
			'conditions' => array(
				'user' => $user
			)
		));

		if (!$userdata) {
			return false;
		}

		$this->user = array(
			'user' => $user,
			'tables' => array(),
			'permissions' => array(
				'table_actions' => array(),
				'actions' => array(),
				'controllers' => array(),
				'extra' => array(),
			)
		);

		foreach ($userdata as $row) {
			if (preg_match('/^[a-z]:[0-9]+:/', $row['value'])) {
				$this->user[$row['type']][$row['name']] = unserialize($row['value']);
			} else {
				$this->user[$row['type']][$row['name']] = $row['value'];
			}
		}

		foreach ($this->user['permissions']['controllers'] as $controller) {
			foreach ($controller as $table) {
				if (!in_array($table, $this->user['tables'])) {
					$this->user['tables'][] = $table;
				}
			}
		}

		return (($this->user['preferences']['password'] == $password) && ($this->user['preferences']['enabled'] == '1'));
	}


	/**
	* function saveActivity (array $activity)
	*
	* Store tables activity
	*/
	public function saveActivity ($activity) {
		global $data;

		$ok = $data->save($data->phpcan_tables['activity'], array(
			'insert' => array(
				'data' => array(
					'user' => $this->user['user'],
					'action' => $activity['action'],
					'table' => $activity['table'],
					'related_id' => $activity['related_id'],
					'comments' => $activity['comments'],
					'date' => date('Y-m-d H:i:s')
				)
			)
		));

		return $ok;
	}
}

$session = new Session;
?>
