<?php
/**
* phpCan v.1
*
* $Id: date.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

include_once ($config->paths['includes'].'class_formats.php');

class FormatDate extends Formats {
	var $format = 'date';

	public function check ($value) {
		global $vars, $debug;

		$this->error = array();

		if (is_array($value)) {
			$value = $value[$this->name];
		}

		$settings = $this->settings[$this->name];

		if (!$settings['required'] && (empty($value) || ($settings['englishformat'] && ($value == '0000-00-00')) || (!$settings['englishformat'] && ($value == '00-00-0000')))) {
			$this->error[$this->name] = __('This field can not be empty');

			return true;
		}

		if ($settings['englishformat'] && preg_match('#^[0-9]{2,4}[/-][0-9]{1,2}[/-][0-9]{1,2}$#', $value)) {
			return true;
		} else if (!$settings['englishformat'] && preg_match('#^[0-9]{1,2}[/-][0-9]{1,2}[/-][0-9]{2,4}$#', $value)) {
			return true;
		} else {
			$this->error[$this->name] = __('The value is not a valid date.');

			return false;
		}
	}

	public function valueDB ($value) {
		global $debug;

		$settings = $this->settings[$this->name];

		preg_match('#^([0-9]{1,4})[/-]([0-9]{1,2})[/-]([0-9]{1,4})$#', $value[$this->name], $parts);

		if ($settings['englishformat']) {
			return array($this->name => ($parts[1].'-'.$parts[2].'-'.$parts[3]));
		} else {
			return array($this->name => ($parts[3].'-'.$parts[2].'-'.$parts[1]));
		}
	}

	public function valueForm ($value) {
		$settings = $this->settings[$this->name];
		$value = $value[$this->name];

		if (empty($value)) {
			$date = array('0000','00','00');
		} else {
			preg_match('#^([0-9]{1,4})[/-]([0-9]{1,2})[/-]([0-9]{1,4})$#', $value, $parts);

			if (strlen($parts[1]) == 4) {
				$date = array($parts[1], $parts[2], $parts[3]);
			} else {
				$date = array($parts[3], $parts[2], $parts[1]);
			}
		}

		if ($settings['englishformat']) {
			return array($this->name => ($date[0].'-'.$date[1].'-'.$date[2]));
		} else {
			return array($this->name => ($date[2].'-'.$date[1].'-'.$date[0]));
		}
	}

	public function settings ($settings) {
		$settings['englishformat'] = isset($settings['englishformat']) ? $settings['englishformat'] : true;

		if (!$settings['default']) {
			$settings['default'] = $settings['englishformat'] ? date('Y-m-d') : date('d-m-Y');
		}

		$this->settings = array(
			$this->name => array(
				'db_type' => 'date',
				'db_default' => '0000-00-00',

				'englishformat' => $settings['englishformat'],
				'default' => $settings['default'],
				'required' => $settings['required'] ? true : false
			)
		);

		return $this->settings;
	}
}
?>
