<?php
/**
* phpCan v.1
*
* $Id: datetime.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

include_once ($config->paths['includes'].'class_formats.php');

class FormatDatetime extends Formats {
	var $format = 'datetime';

	public function check ($value) {
		global $debug;

		$this->error = array();

		if (is_array($value)) {
			$value = $value[$this->name];
		}

		$settings = $this->settings[$this->name];

		if (!$settings['required'] && (empty($value) || ($value == '0000-00-00 00:00:00'))) {
			$this->error[$this->name] = __('This field can not be empty');

			return true;
		}

		if ($settings['englishformat'] == true) {
			$exp = '[0-9]{2,4}[-/][0-9]{1,2}[-/][0-9]{1,2} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}';
		} else {
			$exp = '[0-9]{1,2}[-/][0-9]{1,2}[-/][0-9]{2,4} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}';
		}

		if (preg_match('#^'.$exp.'$#', $value)) {
			return true;
		} else {
			$this->error[$this->name] = __('The value is not a valid date and time.');

			return false;
		}
	}

	public function valueDB ($value) {
		if (is_array($value)) {
			$value = $value[$this->name];
		}

		$settings = $this->settings[$this->name];

		if (empty($value)) {
			return '0000-00-00 00:00:00';
		}

		if ($settings['englishformat'] == true) {
			preg_match('#^([0-9]{2,4})[-/]([0-9]{1,2})[-/]([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2})(:([0-9]{1,2}))?$#', $value, $date);

			return array($this->name => $date[1].'-'.$date[2].'-'.$date[3].' '.$date[4].':'.$date[5].':'.intval(preg_replace('/\W/', '', $date[7])));
		} else {
			preg_match('#^([0-9]{1,2})[-/]([0-9]{1,2})[-/]([0-9]{2,4}) ([0-9]{1,2}):([0-9]{1,2})(:([0-9]{1,2}))?$#', $value, $date);

			return array($this->name => $date[3].'-'.$date[2].'-'.$date[1].' '.$date[4].':'.$date[5].':'.intval(preg_replace('/\W/', '', $date[7])));
		}
	}

	public function valueForm ($value) {
		if (is_array($value)) {
			$value = $value[$this->name];
		}

		$settings = $this->settings[$this->name];

		if (!$settings['englishformat'] && ($value == '0000-00-00 00:00:00')) {
			$value = '00-00-0000 00:00:00';
		}

		preg_match('#^([0-9]{2,4})[-/]([0-9]{1,2})[-/]([0-9]{2,4}) ([0-9]{1,2}):([0-9]{1,2})(:([0-9]{1,2}))?#', $value, $date);

		if ($settings['englishformat'] == true) {
			return array($this->name => $date[1].'-'.$date[2].'-'.$date[3].' '.$date[4].':'.$date[5].':'.intval(preg_replace('/\W/', '', $date[7])));
		} else {
			return array($this->name => $date[3].'-'.$date[2].'-'.$date[1].' '.$date[4].':'.$date[5].':'.intval(preg_replace('/\W/', '', $date[7])));
		}
	}

	public function settings ($settings) {
		$settings['englishformat'] = isset($settings['englishformat']) ? $settings['englishformat'] : true;
		$settings['default'] = $settings['englishformat'] ? date('Y-m-d H:i:s') : date('d-m-Y H:i:s');

		$this->settings = array(
			$this->name => array(
				'db_type' => 'datetime',
				'db_default' => '0000-00-00 00:00:00',

				'englishformat' => $settings['englishformat'],
				'default' => $settings['default'],
				'required' => $settings['required'] ? true : false
			)
		);

		return $this->settings;
	}
}
?>
