<?php
/**
* phpCan v.1
*
* $Id: gmaps.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

include_once ($config->paths['includes'].'class_formats.php');

class FormatGmaps extends Formats {
	var $format = 'gmaps';

	public function check ($value) {
		$this->error = array();

		$x = $this->name.'_x';
		$y = $this->name.'_y';
		$z = $this->name.'_z';

		if ($settings[$x]['required'] && empty($value[$x])) {
			$this->error[$this->name]['x'] = __('This field can not be empty');
			return false;
 		}

		if ($settings[$y]['required'] && empty($value[$y])) {
			$this->error[$this->name]['y'] = __('This field can not be empty');
			return false;
 		}

		if ($settings[$z]['required'] && empty($value[$z])) {
			$this->error[$this->name]['z'] = __('This field can not be empty');
			return false;
 		}

		if (!preg_match('/^[0-9,\.-]*$/', $value[$x])) {
			$this->error[$this->name]['x'] = __('This field haven\'t a valid value');
			return false;
		}

		if (!preg_match('/^[0-9,\.-]*$/', $value[$y])) {
			$this->error[$this->name]['y'] = __('This field haven\'t a valid value');
			return false;
		}

		if (!preg_match('/^[0-9]*$/', $value[$z])) {
			$this->error[$this->name]['z'] = __('This field haven\'t a valid value');
			return false;
		}

		if ($value[$z] < 1 || $value[$z] > 17) {
			$this->error[$this->name]['z'] = __('This field value must be between 1 and 17');
			return false;
		}

		return true;
	}

	public function settings ($settings) {
		$settings['default_x'] = $settings['default_x']?$settings['default_x']:-7.2624778;
		$settings['default_y'] = $settings['default_y']?$settings['default_y']:43.3598840;
		$settings['default_z'] = $settings['default_z']?$settings['default_z']:13;

		$this->settings = array(
			$this->name.'_x' => array(
				'db_type' => 'decimal',
				'db_length_max' => '12,7',
				'db_default' => '0.0000000',

				'subformat' => 'x',
				'default' => $settings['default_x'],
				'required' => $settings['required'] ? true : false
			),
			$this->name.'_y' => array(
				'db_type' => 'decimal',
				'db_length_max' => '12,7',
				'db_default' => '0.0000000',

				'subformat' => 'y',
				'default' => $settings['default_y'],
				'required' => $settings['required'] ? true : false
			),
			$this->name.'_z' => array(
				'db_type' => 'tinyint',
				'db_length_max' => '2',
				'db_default' => 0,
				'db_unsigned' => true,

				'subformat' => 'z',
				'default' => $settings['default_z'],
				'required' => $settings['required'] ? true : false
			),
		);

		return $this->settings;
	}
}
?>
