<?php
/**
* phpCan v.1
*
* $Id: image.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

include_once ($config->paths['includes'].'class_formats.php');

class FormatImage extends Formats {
	var $format = 'image';

	public function check ($value) {
		$this->error = array();

		$settings = $this->settings[$this->name];
		$value = $value[$this->name];

		if ($value['error']) {
			switch ($value['error']) {
				case 1:
					$this->error[$this->name] = __('The uploaded file exceeds the upload_max_filesize directive in php.ini.');
					return false;
				case 2:
					$this->error[$this->name] = __('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form.');
					return false;
				case 3:
					$this->error[$this->name] = __('The uploaded file was only partially uploaded.');
					return false;
				case 6:
					$this->error[$this->name] = __('Missing a temporary folder.');
					return false;
				case 7:
					$this->error[$this->name] = __('Failed to write file to disk.');
					return false;
				case 6:
					$this->error[$this->name] = __('File upload stopped by extension.');
					return false;
			}
		}

		if (!$settings['required'] && ($value['size'] == 0)) {
			return true;
		}

		if ($settings['required'] && ($value['size'] == 0)) {
			$this->error[$this->name] = __('This field can not be empty');

			return false;
		} else if (!recursiveFolder($settings['media_dir'].$settings['dir'])) {
			$this->error[$this->name] = __('The folder to store the file don\'t exists or can\'t be created');

			return false;
		} else if (!is_writable($settings['media_dir'].$settings['dir'])) {
			$this->error[$this->name] = __('The folder to store the file haven\'t writing permissions');

			return false;
		}

		$ext = pathinfo($value['name'], PATHINFO_EXTENSION);

		if (!$ext || ($settings['valid_extensions'] && !in_array(strtolower($ext), $settings['valid_extensions']))) {
			$this->error[$this->name] = __('This file has a invalid extension');

			return false;
		} else if (!@getImageSize($value['tmp_name'])) {
			$this->error[$this->name] = __('This file isn\'t a valid image');

			return false;
		}

		return true;
	}

	public function valueDB ($value) {
		$value = $value[$this->name];

		if (($value['size'] == 0) || !is_file($value['tmp_name'])) {
			$this->newValue = $this->oldValue;

			return array($this->name => $this->newValue);
		}

		$value['name'] = filename($value['name']);
		$settings = $this->settings[$this->name];

		$dir = $settings['media_dir'].$settings['dir'];

		while (is_file($dir.$value['name'])) {
			$value['name'] = rand(0, 9999).'-'.$value['name'];
		}

		if (move_uploaded_file($value['tmp_name'], $dir.$value['name'])) {
			$this->newValue = $settings['dir'].$value['name'];

			$img = getImageSize($dir.$value['name']);

			if (($img[0] > $settings['max_width'])
			|| ($img[1] > $settings['max_height'])
			|| (filesize($dir.$value['name']) > $settings['max_size'])) {
				global $config;

				include_once ($config->paths['includes'].'phpThumb/phpthumb.class.php');

				$phpThumb = new phpThumb();

				$phpThumb->setSourceFilename($dir.$value['name']);
				$phpThumb->setParameter('w', $settings['max_width']);
				$phpThumb->setParameter('h', $settings['max_height']);
				$phpThumb->setParameter('maxb', $settings['max_size']);

				$phpThumb->GenerateThumbnail();
				$phpThumb->RenderToFile($dir.$value['name']);
			}
		} else {
			$this->newValue = $this->oldValue;
		}

		return array($this->name => $this->newValue);
	}

	public function afterSave ($id) {
		if (($this->newValue == $this->oldValue) || empty($this->oldValue)) {
			return true;
		}

		if ($id == 0) {
			$file = end(explode('/', $this->newValue));
		} else {
			$file = end(explode('/', $this->oldValue));
		}

		$settings = $this->settings[$this->name];

		if (is_file($settings['media_dir'].$settings['dir'].$file)) {
			//unlink($settings['media_dir'].$settings['dir'].$file);
		}

		return true;
	}

	public function settings ($settings) {
		if (!$settings['media_dir']) {
			global $config;

			$settings['media_dir'] = $config->paths['scene-uploads'];
			$settings['www-media_dir'] = $config->paths['www-scene-uploads'];
		}

		$this->settings = array(
			$this->name => array(
				'db_type' => 'varchar',
				'db_length_max' => 225,

				'max_size' => $settings['max_size'] ? toBytes($settings['max_size']) : toBytes(ini_get('upload_max_filesize')),
				'valid_extensions' => is_array($settings['valid_extensions']) ? $settings['valid_extensions'] : array('png', 'jpg', 'jpeg'),
				'media_dir' => $settings['media_dir'],
				'www-media_dir' => $settings['www-media_dir'],
				'dir' => $settings['dir'] ? $settings['dir'].'/' : $this->name.'/',
				'required' => $settings['required'] ? true : false
			)
		);

		return $this->settings;
	}
}
?>
