<?php
/**
* phpCan v.1
*
* $Id: class_db.php 360 2009-08-07 11:46:27Z oscar $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

require_once ($config->paths['includes'].'class_'.$config->db['type'].'.php');

class DB extends DataBase {
	var $last_id = array();
	var $paginate = array();
	var $error = array();

	var $operations = array();
	var $result = array();
	var $result_limits = array();

	var $phpcan_tables = array();

	/**
	* function seTables (void)
	*
	* Recover tables data and assign the default values
	* to complete all settings
	*/
	protected function setTables () {
		global $config, $debug;

		$tables = $fields = $tables_info = array();

		if (!class_exists('FormatId')) {
			include_once ($config->paths['formats'].'id.php');
		}

		$pre = $config->db['preffix'];

		$this->phpcan_tables = array(
			'relations' => $pre.'relations',
			'admin' => $pre.'admin',
			'activity' => $pre.'activity'
		);

		//Relations table
		$config->tables[$this->phpcan_tables['relations']] = array(
			'tables' => array(
				'format' => 'varchar',
				'length_max' => 50,
				'key' => 'relations'
			),
			'id1' => array(
				'format' => 'integer',
				'length_max' => 10,
				'key' => 'relations'
			),
			'id2' => array(
				'format' => 'integer',
				'length_max' => 10,
				'key' => 'relations'
			)
		);

		//Admin table
		$config->tables[$this->phpcan_tables['admin']] = array(
			'type' => 'varchar',
			'name' => 'varchar',
			'value' => 'text',
			'user' => 'varchar'
		);

		//Activity table
		$config->tables[$this->phpcan_tables['activity']] = array(
			'user' => 'varchar',
			'action' => 'varchar',
			'table' => 'varchar',
			'related_id' => 'varchar',
			'comments' => 'text',
			'date' => 'datetime'
		);

		foreach ($config->tables as $table => $info) {
			$tables[$table]['id'] = new FormatId($table, 'id');
			$fields[$table]['id'] = 'id';

			foreach ($info as $field => $settings) {
				if ($field == 'id') {
					continue;
				}

				if (!is_array($settings)) {
					$settings = array('format' => $settings);
				}

				if (!is_file($config->paths['formats'].$settings['format'].'.php')) {
					continue;
				}

				$classname = 'Format'.ucfirst($settings['format']);

				if (!class_exists($classname)) {
					include_once ($config->paths['formats'].$settings['format'].'.php');
				}

				$tables[$table][$field] = new $classname($table, $field, $settings);

				foreach (array_keys($tables[$table][$field]->settings) as $dbfield) {
					$fields[$table][$dbfield] = $tables[$table][$field]->name;
				}
			}
		}

		foreach ($config->tables_info as $table => $info) {
			$tables_info[$table] = $info;
		}

		$config->tables = $tables;
		$config->fields = $fields;
		$config->tables_info = $tables_info;
	}
	
	/**
	* public function groupFields (string $table, array $fields, [string $suffix])
	*
	* return array
	*/
	public function groupFields ($table, $fields, $suffix = '', $include_relation_fields = true) {
		if (!is_array($fields)) {
			return array();
		}

		global $config;

		$db_values = array();
		$sorted_array = array();

		foreach ($config->fields[$table] as $field => $v) {
			if (array_key_exists($field.$suffix, $fields)) {
				$db_values[$field] = $fields[$field.$suffix];
			}
		}

		foreach ($db_values as $field => $value) {
			$sorted_array[$config->fields[$table][$field]][$field.$suffix] = $value;
		}

		if ($include_relation_fields) {
			foreach ((array)$config->relations[$table] as $table_relation) {
				foreach ($table_relation as $relation) {
					if ($relation['mode'] == 'x 1' && array_key_exists($relation['join'], $fields)) {
						$sorted_array[$relation['join']] = $fields[$relation['join']];
					}
				}
			}
		}

		return $sorted_array;
	}

	/**
	* function setRelations (void)
	*
	* Create fields relations
	*/
	protected function setRelations () {
		global $config;

		if (count($config->relations) == 0) {
			$config->relations = array();

			return false;
		}

		$relations = $config->relations;

		$config->relations = array();

		foreach ($relations as $each) {
			list($table1, $table2) = explode(' ', $each['tables']);

			switch ($each['mode']) {
				case 'x 1':
				$join1 = $join2 = empty($each['name']) ? 'id_'.$table2 : $each['name'];
				$relation_table = '';
				break;

				case '1 x':
				$join1 = $join2 = empty($each['name']) ? 'id_'.$table1 : $each['name'];
				$relation_table = '';
				break;

				case '1 1':
				$join1 = 'id_'.$table2;
				$join2 = 'id_'.$table1;
				$relation_table = '';
				break;

				case 'x x':
				$join1 = $table1 < $table2 ? $table1.' '.$table2 : $table2.' '.$table1;
				if (!empty($each['name'])) {
					$join1 .= ' '.$each['name'];
				}
				$join2 = $join1;
				$relation_table = $each['relation_table'] ? $each['relation_table'] : ($config->db['preffix'].'relations');
				break;
			}

			$config->relations[$table1][$table2][] = array(
				'dependent' => $each['dependent'],
				'name' => $each['name'],
				'mode' => $each['mode'],
				'join' => $join1,
				'relation_table' => $relation_table
			);

			$config->relations[$table2][$table1][] = array(
				'dependent' => $each['dependent'],
				'name' => $each['name'],
				'mode' => preg_replace('/^(1|x) (1|x)$/', '$2 $1', $each['mode']),
				'join' => $join2,
				'relation_table' => $relation_table
			);

			if ($relation_table && $relation_table != $this->phpcan_tables['relations']) {
				$config->relations[$relation_table][$table1][] = array(
					'dependent' => $each['dependent'],
					'name' => $each['name'],
					'mode' => '1x x',
					'join' => $join1,
					'related_table' => $table2
				);

				$config->relations[$relation_table][$table2][] = array(
					'dependent' => $each['dependent'],
					'name' => $each['name'],
					'mode' => '1x x',
					'join' => $join1,
					'related_table' => $table1
				);

				$config->relations[$table1][$relation_table][] = array(
					'dependent' => $each['dependent'],
					'name' => $each['name'],
					'mode' => 'x x1',
					'join' => $join1,
					'related_table' => $table2
				);

				$config->relations[$table2][$relation_table][] = array(
					'dependent' => $each['dependent'],
					'name' => $each['name'],
					'mode' => 'x x1',
					'join' => $join1,
					'related_table' => $table1
				);
			}
		}
	}

	/**
	* function multipleQueries (string $q)
	*
	* Execute multiple queries at time
	*/
	public function multipleQueries ($q) {
		$result = array();

		foreach (explode(';', $q) as $query) {
			$query = trim(str_replace(array("\n","\r"), '', $query));

			if (empty($query)) {
				continue;
			}

			$ok = $this->query($query.';');

			$result[] = array($ok, $this->query);
		}

		return $result;
	}

	/*
	* function tableExists (string $table)
	*
	* Return boolean
	*/
	public function tableExists ($table) {
		if (!$table) {
			return false;
		}

		global $config;

		if (isset($config->tables[$table])) {
			return $table;
		}

		if (isset($this->phpcan_tables[$table])) {
			return $this->phpcan_tables[$table];
		}

		if (in_array($table, $this->phpcan_tables)) {
			return $table;
		}

		return false;
	}

	/*
	* function checkData (string $table, array $data)
	* 
	* Check data before insert or update in database.
	*
	* Return array. On error, return false
	*/
	public function checkData ($table, $data) {
		global $config, $debug;

		if (!$config->tables[$table] || !$data) {
			return false;
		}

		//Is numerical array?
		$keys = implode('', array_keys($data));
		$sub_array = current($data);

		if (preg_match('/^[0-9]*$/', $keys) && is_array($sub_array)) {
			$result_data = array();

			foreach ($data as $key => $value) {
				$result_data[$key] = $this->checkData($table, $value);
			}

			return count($this->error) ? false : $result_data;
		}

		$fields = $this->groupFields($table, $data);

		$data = array();

		foreach ((array)$fields as $field => $values) {
			$object = $config->tables[$table][$field];

			if ($object->check($values)) {
				$values = $object->valueDB($values);

				if (is_array($values)) {
					$data = array_merge($data, $values);
				}
			} else {
				$this->error[$table][$field] = $object->error;
			}
		}

		return count($this->error) ? false : $data;
	}

	
	/**
	* function save (string $table, array $operations)
	*
	* Modifies dates in database
	*/
	public function save ($table, $operations = array()) {
		global $config, $debug, $vars;

		$this->connect();

		$savedIds = array();
		$toRelate = array();
		$toUnrelate = array();

		//Analyce operations
		foreach ($operations as $type => $data) {

			//Define relate and unrelate options
			if (!$data['relate']) {
				$data['relate'] = array();
			}
			if (!$data['unrelate']) {
				$data['unrelate'] = array();
			}
			if ($data['relate'] && !is_int(key($data['relate']))) {
				$data['relate'] = array($data['relate']);
			}
			if ($data['unrelate'] && !is_int(key($data['unrelate']))) {
				$data['unrelate'] = array($data['unrelate']);
			}
			
			//Analice if conditions use external tables
			foreach ((array)$data['conditions'] as $condition => $v) {
				preg_match('#^((\w+)\.)?(\w+) ?(\((\w+)?\))?(.*)$#', $condition, $t);

				if ($t[2] && $t[2] != $table) {
					$ids = $this->select($table, array(
						'fields' => 'id',
						'conditions' => $data['conditions'],
						'limit' => $data['limit']
					));

					if ($data['limit'] == 1) {
						$ids = $ids['id'];
					} else {
						foreach ($ids as $k => $v) {
							$ids[$k] = $v['id'];
						}
					}

					if (!$ids) {
						return true;
					}

					$data['conditions'] = array('id' => $ids);
					break;
				}
			}

			//Execute main operations
			switch ($type) {

				//Insert/update operation
				case 'insert':
				case 'update':
				case 'replace':
					if (is_array($data['overwrite_control']) && ($type == 'update')) {
						$rows = $this->select($table, array(
							'fields' => array_keys($data['overwrite_control']),
							'conditions' => $data['conditions'],
							'limit' => 1
						));

						if (count($rows) == 0) {
							continue;
						}

						foreach ($rows as $dbfield => $value) {
							$field = $config->fields[$table][$dbfield];
							$config->tables[$table][$field]->oldValue = $rows[$field];
						}
					}

					$data['data'] = $this->checkData($table, $data['data']);

					if (!$data['data']) {
						return false;
					}

					//Find for 'x 1' relations
					$relate_fields = array();

					foreach ($data['relate'] as $k => $relate) {
						$relation = $this->getRelation($table, $relate['table'], $relate['name']);

						//Add the 'x 1' relations with id condition to insert data (and remove it)
						if ($relation['mode'] == 'x 1') {
							if (count($relate['conditions']) == 1 && $relate['conditions']['id']) {
								$id = $relate['conditions']['id'];
							} else {
								$id = $this->select($relate['table'], array(
									'fields' => 'id',
									'conditions' => $relate['conditions'],
									'limit' => 1
								));
								$id = $id['id'];
							}

							if ($id) {
								$data['data'][$relation['join']] = $id;
								$relate_fields[] = $relation['join'];
							}

							unset($data['relate'][$k]);
						}
					}

					if (($type == 'insert') || ($type == 'replace')) {
						//Insert the register
						if (($num = $this->$type($table, $data)) === false) {
							$debug->db(__('There was an error in the insert operation'));
							return false;
						} else {
							$last_id = $this->last_id();
						}

						//Recovery id
						if ($data['saveId'] || $data['relate']) {
							if ($num != 1) {
								$this->select($table, array(
									'fields' => 'id',
									'conditions' => $data['data']
								));
								$debug->e('Facer que as condicións en subarrays funcionen como OR');
							}
						}

						//remove unrelate option (not necessary at insert)
						$data['unrelate'] = array();

					} else {
						if (is_array($data['overwrite_control'])) {
							foreach ($data['data'] as $dbfield => $value) {
								if (in_array($dbfield, $relate_fields)) {
									continue;
								}

								$delete = false;

								if ($dbfield == 'id' || !isset($data['overwrite_control'][$dbfield])) {
									$delete = true;
								} else if ($data['overwrite_control'][$dbfield] == $data['data'][$dbfield]) {
									$field = $config->fields[$table][$dbfield];
									$delete = true;

									if ($field != $dbfield) {
										foreach ($config->tables[$table][$field]->settings as $dbfield2 => $object) {
											if (isset($data['data'][$dbfield2]) && ($data['overwrite_control'][$dbfield2] != $data['data'][$dbfield2])) {
												$delete = false;
											}
										}
									}
								}

								if ($delete) {
									unset($data['data'][$dbfield], $operations[$operation]['data'][$dbfield]);

									continue;
								}

								if (stripslashes($data['overwrite_control'][$dbfield]) != stripslashes($rows[$dbfield])) {
									$this->error[$table][$dbfield][] = __('Field value was edited before yours changes!');

									return false;
								}

								$field = $config->fields[$table][$dbfield];

								$config->tables[$table][$field]->oldValue = $rows[$field];
							}
						}

						unset($data['overwrite_control']);

						$last_id = intval($data['conditions']['id'] ? $data['conditions']['id'] : $data['conditions'][$table.'.id']);

						if (is_array($data['data']) && count($data['data'])) {
							//Update the register
							if ($this->update($table, $data) === false) {
								$debug->db(__('There was an error in the update operation'));
								return false;
							}
						}

						//Recovery id
						if ($data['saveId'] || $data['relate'] || $data['unrelate']) {
							if ($last_id == 0) {
								return false;
							}
						}
					}

					break;

				//Delete operations
				case 'delete':
					if (!$data['conditions']) {
						$debug->db(__('No conditions to delete in %s', $table));
					}

					//Recovery id
					if ((count($data['conditions']) == 1) && ($data['contitions']['id'] || $data['conditions'][$table.'.id'])) {
						$last_id = $data['contitions']['id']?$data['contitions']['id']:$data['conditions'][$table.'.id'];
					} else {
						$last_id = $this->select($table, array(
							'fields' => 'id',
							'conditions' => $data['conditions']
						));

						foreach ($last_id as $k => $v) {
							$last_id[$k] = $v['id'];
						}
					}

					//Unrelate relations
					foreach ((array)$config->relations[$table] as $table_related => $relations) {
						foreach ($relations as $relation) {
							
							//Delete dependent relations
							if ($relation['dependent'] && $relation['mode'] == '1 x') {
								$this->save($table_related, array(
									'delete' => array(
										'conditions' => array(
											$table.'.id' => $last_id
										)
									)
								));

							//Unrelate relations
							} else if ($relation['mode'] != 'x 1') {
								$data['unrelate'][] = array(
									'table' => $table_related,
									'name' => $relation['name'],
								);
							}
						}
					}

					//Remove relate option (not necessary at delete)
					$data['relate'] = array();

					//Delete the register
					if ($this->delete($table, $data) === false) {
						$debug->db(__('There was an error in the update operation'));
						return false;
					}

					break;

				//Relate/unrelate operations
				case 'relate':
				case 'unrelate':
					$alldata = $data;

					//Remove relate/unrelate options (not necessary)
					$data['relate'] = array();
					$data['unrelate'] = array();

					if (!is_integer(key($alldata))) {
						$alldata = array($alldata);
					}
					unset($alldata['relate']);
					unset($alldata['unrelate']);

					foreach ($alldata as $curr) {

						//Id must be defined
						if (!$curr['id']) {
							$debug->db(__('Id must be defined in direct relates/unrelates'));
							continue;
						}

						//Build the relate/unrelate operation
						$data[$type][] = array(
							'table' => $curr['table'],
							'name' => $curr['name'],
							'conditions' => $curr['conditions'],
							'id' => $curr['id']
						);
					}
					break;

					default:
					$debug->db(__('The %s operation is not valid', $type));
					return false;
			}

			//Save id
			if ($data['saveId']) {
				$savedIds[$data['saveId']] = $last_id;
			}

			//Save the relate options
			foreach ($data['relate'] as $relate) {
				$toRelate[] = array(
					'table' => $relate['table'],
					'conditions' => $relate['conditions'],
					'name' => $relate['name'],
					'ids' => $relate['id'] ? $relate['id'] : $last_id
				);
			}

			//Save the unrelate options
			foreach ($data['unrelate'] as $unrelate) {
				$toUnrelate[] = array(
					'table' => $unrelate['table'],
					'conditions' => $unrelate['conditions'],
					'name' => $unrelate['name'],
					'ids' => $unrelate['id'] ? $unrelate['id'] : $last_id
				);
			}

			if (count($this->error) == 0) {
				$dir = $vars->isAdmin() ? 'admin-events' : 'scene-events';

				if (is_array($config->events[$table][$type])) {
					foreach ($config->events[$table][$type] as $event) {
						include ($config->paths[$dir].$event.'.php');
					}
				} else if (isset($config->events[$table][$type])) {
					include ($config->paths[$dir].$config->events[$table][$type].'.php');
				}
			}
		}

		//Execute the relate operations
		foreach ($toRelate as $relate) {
			$relation = $this->getRelation($table, $relate['table'], $relate['name']);

			//Analice relation type
			switch($relation['mode']) {

				case '1 x':
					//Build the query to update the relation
					$query = array(
						'data' => array(
							$relation['join'] => $relate['ids']
						),
						'conditions' => $relate['conditions']
					);

					if ($this->update($relate['table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}

					break;

				case 'x 1':
					//Select the id to relate
					if (count($relate['conditions']) == 1 && $relate['conditions']['id']) {
						$id = $relate['conditions']['id'];
					} else {
						$id = $this->select($relate['table'], array(
							'fields' => 'id',
							'conditions' => $relate['conditions']
						));
						$id = $id['id'];
					}

					//Build the query to update the relation
					$query = array(
						'data' => array(
							$relation['join'] => $id
						),
						'conditions' => array(
							'id' => $relate['ids']
						)
					);

					if ($this->update($table, $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}

					break;

				case 'x x':
					//Build the query to insert the relation
					if ($table < $relate['table']) {
						$id1 = 'id1';
						$id2 = 'id2';
					} else {
						$id1 = 'id2';
						$id2 = 'id1';
					}

					$query = array(
						'data' => array(
							'tables' => $relation['join'],
							$id1 => $relate['ids'],
							$id2 => $id
						)
					);

					//Select the id to relate
					if (count($relate['conditions']) == 1 && $relate['conditions']['id']) {
						$query['data'][$id2] = $relate['conditions']['id'];
					} else if ($relate['conditions']) {
						$id = $this->select($relate['table'], array(
							'fields' => 'id',
							'conditions' => $relate['conditions']
						));

						foreach ($id as $k => $v) {
							$id[$k] = $v['id'];
						}

						$query['data'][$id2] = $id;
					} else {
						$debug->db(__('Conditions must be to relate operation'));

						return false;
					}

					if ($this->insert($relation['relation_table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}
					break;

				case '1 1':
					//Select the id to relate
					if (count($relate['conditions']) == 1 && $relate['conditions']['id']) {
						$id = $relate['conditions']['id'];
					} else {
						$id = $this->select($relate['table'], array(
							'fields' => 'id',
							'conditions' => $relate['conditions'],
							'limit' => 1
						));
						$id = $id['id'];
					}

					//Build the query to update the relation
					$inverse_join = $this->getRelation($relate['table'], $table, $relate['name']);
					$inverse_join = $inverse_join['join'];

					$query = array(
						'data' => array(
							$relation['join'] => $id
						),
						'conditions' => array(
							'id' => $relate['ids']
						)
					);

					if ($this->update($table, $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}

					$query = array(
						'data' => array(
							$inverse_join => $relate['ids']
						),
						'conditions' => array(
							'id' => $id
						)
					);

					if ($this->update($relate['table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}
					break;

				default:
				$debug->db(__('There is not relations between the tables "%s" and "%s"', $table, $relate['table']));
			}
		}

		//Execute the unrelate operations
		foreach ($toUnrelate as $unrelate) {
			$relation = $this->getRelation($table, $unrelate['table'], $unrelate['name']);

			//Analice relation type
			switch($relation['mode']) {

				case '1 x':
					//Build the query to update the relation
					if (!$unrelate['conditions']) {
						$unrelate['conditions'] = array();
					}
					$unrelate['conditions'][$relation['join']] = $unrelate['ids'];

					$query = array(
						'data' => array(
							$relation['join'] => 0
						),
						'conditions' => $unrelate['conditions']
					);

					if ($this->update($unrelate['table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}

					break;

				case 'x 1':
					//Select the id to unrelate
					if (count($unrelate['conditions']) == 1 && $unrelate['conditions']['id']) {
						$id = $unrelate['conditions']['id'];
					} else {
						$id = $this->select($unrelate['table'], array(
							'fields' => 'id',
							'conditions' => $unrelate['conditions']
						));
						$id = $id['id'];
					}

					//Build the query to update the relation
					$query = array(
						'data' => array(
							$relation['join'] => 0
						),
						'conditions' => array(
							'id' => $unrelate['ids']
						)
					);
					
					if ($unrelate['conditions']) {
						$query['conditions'][$relation['join']] = $id;
					}

					if ($this->update($table, $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}
					break;

				case 'x x':
					//Build the query to insert the relation
					if ($table < $unrelate['table']) {
						$id1 = 'id1';
						$id2 = 'id2';
					} else {
						$id1 = 'id2';
						$id2 = 'id1';
					}

					$query = array(
						'conditions' => array(
							'tables' => $relation['join'],
							$id1 => $unrelate['ids']
						)
					);

					//Select the id to unrelate
					if (count($unrelate['conditions']) == 1 && $unrelate['conditions']['id']) {
						$query['conditions'][$id2] = $unrelate['conditions']['id'];
					} else if ($unrelate['conditions']) {
						$id = $this->select($unrelate['table'], array(
							'fields' => 'id',
							'conditions' => $unrelate['conditions']
						));

						foreach ($id as $k => $v) {
							$id[$k] = $v['id'];
						}

						$query['conditions'][$id2] = $id;
					}

					//Unrelate
					if ($this->delete($relation['relation_table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}
					break;

				case '1 1':
					//Select the id to relate
					if (count($unrelate['conditions']) == 1 && $unrelate['conditions']['id']) {
						$id = $unrelate['conditions']['id'];
					} else {
						$id = $this->select($unrelate['table'], array(
							'fields' => 'id',
							'conditions' => $unrelate['conditions'],
							'limit' => 1
						));
						$id = $id['id'];
					}

					//Build the query to update the relation
					$inverse_join = $this->getRelation($unrelate['table'], $table, $unrelate['name']);
					$inverse_join = $inverse_join['join'];

					$query = array(
						'data' => array(
							$relation['join'] => 0
						),
						'conditions' => array(
							'id' => $unrelate['ids']
						)
					);

					if ($this->update($table, $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}

					$query = array(
						'data' => array(
							$inverse_join => 0
						),
						'conditions' => array(
							'id' => $id
						)
					);

					if ($this->update($unrelate['table'], $query) === false) {
						$debug->db(__('There was an error in the update operation'));
					}
					break;

				default:
				$debug->db(__('There is not relations between the tables "%s" and "%s"', $table, $unrelate['table']));
			}
		}

		return $savedIds;
	}


	public function getRelation ($table1, $table2, $name = '') {
		if (!$table1 || !$table2) {
			return false;
		}

		if (!$name) {
			$name = '';
		}

		global $config, $debug;

		$relation = $config->relations[$table1][$table2];

		if ($relation) {
			foreach ($relation as $rel) {
				if ($rel['name'] == $name) {
					return $rel;
				}
			}
		}

		return false;
	}


	public function autoFields ($table, $fields) {
		global $config, $debug;
		
		if (!$table || !$fields) {
			return false;
		}

		if (!$config->tables[$table]) {
			$debug->db(__('The table %s does not exists', $table));

			return false;
		}

		if (($fields == '*') || ($fields[0] == '*')) {
			$fields = array();

			//Table fields
			foreach (array_keys($config->fields[$table]) as $dbfield) {
				$fields[] = $dbfield;
			}

			//Relation fields
			if (is_array($config->relations[$table])) {
				foreach ($config->relations[$table] as $table_relation) {
					foreach ($table_relation as $relation) {
						if ($relation['mode'] == 'x 1') {
							$fields[] = $relation['join'];
						}
					}
				}
			}
		} else {
			if (is_string($fields)) {
				$tmpfields = array();

				foreach (explode(',', $fields) as $field) {
					$field = trim($field);
					$tmpfields[] = $field;
				}

				$fields = $tmpfields;
			}
		}

		return $fields;
	}
	
	public function selectFields ($table, $fields) {
		global $config, $debug;

		if (($fields == '*') || ($fields[0] == '*')) {
			$auto_fields = true;
		} else {
			$auto_fields = false;
		}

		$fields = $this->autoFields($table, $fields);

		if (!$fields) {
			return array();
		}

		if (!$fields['id'] && !in_array('id', $fields) && isset($config->fields[$table]['id'])) {
			$fields['id'] = 'id';
		}

		$selectFields = array();

		foreach ($fields as $oldName => $newName) {
			$name = is_int($oldName) ? $newName : $oldName;

			if ($config->fields[$table][$name] || $auto_fields) {
				if ($name == $newName) {
					$selectFields[$name] = $newName;
				} else {
					$selectFields[$oldName] = $newName;
				}
			} else if ($config->tables[$table][$name]) {
				if ($name == $newName) {
					$more_fields = array_keys($config->tables[$table][$name]->settings);
				} else {
					$more_fields = array();

					foreach ($config->tables[$table][$name]->settings as $dbfield => $settings) {
						$more_fields[$dbfield] = $newName.'_'.$settings['subformat'];
					}
				}

				$selectFields = array_merge($selectFields, $more_fields);
			} else if ($this->fieldExists($table, $name)) {
				$selectFields[$name] = $name;
			} else {
				$debug->db(__('The field "%s" does not exists', $name));
			}
		}

		return $selectFields;
	}


	public function fieldExists ($table, $field, $as_format = false) {
		global $config;

		if ($config->fields[$table][$field]) {
			return true;
		}

		if ($as_format && $config->tables[$table][$field]) {
			return true;
		}

		foreach ($config->relations[$table] as $table_relation) {
			foreach ($table_relation as $relation) {
				if ($relation['join'] == $field) {
					return true;
				}
			}
		}

		return false;
	}


	public function select ($table, $operations, $store = '') {
		global $debug;

		$this->connect();

		if (!$operations['fields']) {
			$debug->db(__('There is not fields to select'));
		}

		if (is_array($operations['pagination'])) {
			return $this->selectPaginate($table, $operations, $store);
		}

		//Update internal variables
		$this->operations = $operations;
		$this->result_limits = array();
		$this->result = array();

		//Make select
		$this->makeSelect($table, $this->operations);

		//Save results
		$result = $this->saveSelect($table, $this->operations);

		if ($this->result_limits['.'.$table.'.']['limit'] == 1) {
			$result = current($result);
		}

		if ($store) {
			$this->register($store, $result);
		}

		return $result;
	}


	public function selectCount ($table, $operations = array()) {
		$this->connect();

		unset($operations['addTables']);
		unset($operations['sort']);
		unset($operations['limit']);
		unset($operations['paginate']);
		unset($operations['fields']);

		$operations['fieldsCommands'] = 'COUNT(*)';

		//Make select
		$this->makeSelect($table, $operations);

		if (is_array($this->result['.'.$table.'.'])) {
			return intval(current(current($this->result['.'.$table.'.'])));
		} else {
			return 0;
		}
	}

	private function selectPaginate ($table, $operations, $store = '') {
		global $debug;

		if ($operations['limit'] == 0) {
			$operations['limit'] = 10;
		}

		$page = intval($operations['pagination']['page']);
		$map = intval($operations['pagination']['map']);
		$pagination_store = $operations['pagination']['store'] ? $operations['pagination']['store'] : 'pagination';

		unset($operations['pagination']);

		$total = $this->selectCount($table, $operations);

		$this->pagination['page'] = $page;
		$this->pagination['total'] = $total;
		$this->pagination['total_pages'] = ceil($this->pagination['total'] / $operations['limit']);

		if (($this->pagination['page'] < 1) || ($this->pagination['page'] > $this->pagination['total_pages'])) {
			$this->pagination['page'] = 1;
		}

		if ($map == 0) {
			$map = 10;
		}

		$interval = intval($map / 2);

		$this->pagination['first'] = $this->pagination['page'] - $interval;
		$this->pagination['last'] = $this->pagination['page'] + $interval;

		if ($this->pagination['first'] < 1) {
			$this->pagination['first'] = 1;
			$this->pagination['last'] = $map;
		}

		if ($this->pagination['last'] > $this->pagination['total_pages']) {
			$this->pagination['last'] = $this->pagination['total_pages'];
			$this->pagination['first'] = $this->pagination['total_pages'] - $map;

			if ($this->pagination['first'] < 1) {
				$this->pagination['first'] = 1;
			}
		}

		while (($this->pagination['last'] - $this->pagination['first']) >= $map) {
			if (($this->pagination['page'] - $this->pagination['first']) > $interval) {
				$this->pagination['first']++;
			} else {
				$this->pagination['last']--;
			}
		}

		$this->pagination['previous'] = (($this->pagination['page'] - 1) < 1)?false:($this->pagination['page'] - 1);
		$this->pagination['next'] = (($this->pagination['page'] + 1) > $this->pagination['total_pages'])?false:($this->pagination['page'] + 1);

		$operations['limit'] = array(($this->pagination['page'] * $operations['limit']) - $operations['limit'], $operations['limit']);

		$this->register($pagination_store, $this->pagination);

		return $this->select($table, $operations, $store);
	}


	private function makeSelect ($table, $data, $prevTable = '', $arrayResult = '') {
		global $debug;

		$query = array();
		
		//If there is a prevTable
		if ($prevTable && $arrayResult) {

			//If result is empty
			if (!$this->result[$arrayResult]) {
				$arrayResult = $prevTable.'.'.$table.'.'.$data['relation_name'];
				$this->result[$arrayResult] = array();

				foreach((array)$data['addTables'] as $newData) {
					$newTable = $newData['table'];
					$this->makeSelect($newTable, $newData, $table, $arrayResult);
				}

				return false;
			}

			//If result is not empty, get the ids
			$ids = array();
			foreach ($this->result[$arrayResult] as $n => $row) {
				$ids[] = $row['id'];
			}

			//Add the condition to the select
			$conditionToPrev = $prevTable.'.id';
			if ($data['relation_name']) {
				$conditionToPrev .= ' ('.$data['relation_name'].')';
			}
			$data['conditions'][$conditionToPrev] = $ids;

			unset ($ids);
		}

		//Define $arrayResult
		$arrayResult = $prevTable.'.'.$table.'.'.$data['relation_name'];

		//Get fields names
		$query['fields'][$table] = $this->selectFields($table, $data['fields']);

		//Proccess conditions type "and" & "or"
		$conditions = array('conditions', 'conditions_or', 'conditions_and');

		$new_key_condition = 1000;
		foreach ($conditions as $condition) {
			$new_key_condition++;

			$conditions_type = ($condition == 'conditions') ? 'conditions_and' : $condition;

			if (!$data[$condition]) {
				continue;
			}

			//Proccess conditions
			foreach ((array)$data[$condition] as $kCondition => $vCondition) {
				$new_key_condition++;

				if (is_int($kCondition) && is_array($vCondition)) {
					$parenthesis = true;
				} else {
					$vCondition = array($kCondition => $vCondition);
					$parenthesis = false;
				}

				//Proccess subconditions
				foreach ($vCondition as $keyCondition => $valueCondition) {
					$new_key_condition++;

					//If is literal condition
					if (is_int($keyCondition)) {
						if ($parenthesis) {
							$query[$conditions_type][$kCondition][] = $valueCondition;
						} else {
							$query[$conditions_type][] = $valueCondition;
						}

						continue;
					}

					preg_match('#^([0-9]+ )?((\w+)\.)?(\w+) ?(\((\w+)?\))?(.*)$#', $keyCondition, $t);

					$relationTable = $t[3];
					$fieldName = $t[4];
					$relationName = $t[6];
					$conditionType = trim($t[7]);

					unset ($t);

					if (!$fieldName && !is_int($keyCondition)) {
						continue;
					}

					if ($conditionType) {
						$conditionType = ' '.$conditionType;
					}

					//If there is not a relation table
					if (!$relationTable || ($relationTable == $table)) {
						if ($parenthesis) {
							$query[$conditions_type][$kCondition]["$table.$fieldName$conditionType"] = $valueCondition;
						} else {
							$query[$conditions_type]["$table.$fieldName$conditionType"] = $valueCondition;
						}

						continue;
					}

					//If there is a relation table
					$relationInfo = $this->getRelation($table, $relationTable, $relationName);
					$relationJoin = $relationInfo['join'];

					//Build query as relation type
					switch ($relationInfo['mode']) {
						case '1 x':
							$query['fields'][$relationTable] = array();

							if ($parenthesis) {
								$query[$conditions_type][$kCondition]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								$query[$conditions_type][$kCondition][$new_key_condition] = "$table.id = $relationTable.$relationJoin";
							} else {
								$query[$conditions_type]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								$query[$conditions_type][$new_key_condition] = "`$table`.id = `$relationTable`.$relationJoin";
							}

							//Add field related_with_*
							if ($keyCondition == $conditionToPrev) {
								$query['fields'][$relationTable]['id'] = 'related_with_'.$prevTable;
							}

							//Unique value
							$this->result_limits[$arrayResult]['start'] = 0;
							$this->result_limits[$arrayResult]['limit'] = 1;

							break;

						case 'x 1':

							if ($fieldName == 'id') {
								if ($parenthesis) {
									$query[$conditions_type][$kCondition]["$table.$relationJoin$conditionType"] = $valueCondition;
								} else {
									$query[$conditions_type]["$table.$relationJoin$conditionType"] = $valueCondition;
								}
							} else {
								$query['fields'][$relationTable] = array();
								$query[$conditions_type]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								$query[$conditions_type][$new_key_condition] = "`$table`.$relationJoin = $relationTable.id";
							}

							//Add field related_width_*
							if ($keyCondition == $conditionToPrev) {
								$query['fields'][$table][$relationJoin] = 'related_with_'.$prevTable;
							}

							break;

						case 'x x':
							$relations = $relationInfo['relation_table'];

							if ($table < $relationTable) {
								$id1 = 'id1';
								$id2 = 'id2';
							} else {
								$id1 = 'id2';
								$id2 = 'id1';
							}

							if (!$query['fields'][$relations]) {
								$query['fields'][$relations] = array();
							}
							
							//Add join where (out of parenthesis)
							$query[$conditions_type]["$relations.tables"] = $relationJoin;

							if (!in_array("$table.id = $relations.$id1", (array)$query[$conditions_type])) {
								$query[$conditions_type][$new_key_condition] = "`$table`.id = `$relations`.$id1";
								$new_key_condition++;
							}

							if ($fieldName != 'id') {
								if (!isset($query['fields'][$relationTable])) {
									$query['fields'][$relationTable] = array();
								}

								if (!in_array("$relationTable.id = $relations.$id2", (array)$query[$conditions_type])) {
									$query[$conditions_type][$new_key_condition] = "`$relationTable`.id = `$relations`.$id2";
									$new_key_condition++;
								}
							}

							//Add the rest of conditions
							if ($parenthesis) {
								if ($fieldName == 'id') {
									$query[$conditions_type][$kCondition]["$relations.$id2$conditionType"] = $valueCondition;
								} else {
									$query[$conditions_type][$kCondition]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								}
							} else {
								if ($fieldName == 'id') {
									$query[$conditions_type]["$relations.$id2$conditionType"] = $valueCondition;
								} else {
									$query[$conditions_type]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								}
							}

							//Add field related_with_*
							if ($keyCondition == $conditionToPrev) {
								$query['fields'][$relations][$id2] = 'related_with_'.$prevTable;
							}

							break;

						case '1x x':
							$id = ($relationInfo['related_table'] < $relationTable) ? 'id2' : 'id1';

							$query[$conditions_type]["$table.tables"] = $relationInfo['join'];
							$query[$conditions_type]["$table.$id"] = $valueCondition;

							//Add field relatd_with_*
							if ($keyCondition == $conditionToPrev) {
								$query['fields'][$table][$id] = 'related_with_'.$prevTable;
							}

							//Unique value
							$this->result_limits[$arrayResult]['start'] = 0;
							$this->result_limits[$arrayResult]['limit'] = 1;

							break;

						case '1 1':
							$query['fields'][$relationTable] = array();

							$inverse_join = $this->getRelation($relationTable, $table, $relationName);
							$inverse_join = $inverse_join['join'];

							if ($fieldName == 'id') {
								if ($parenthesis) {
									$query[$conditions_type][$kCondition]["$table.$relationJoin$conditionType"] = $valueCondition;
									$query[$conditions_type][$kCondition][] = "$table.id = $relationTable.$inverse_join";
								} else {
									$query[$conditions_type]["$table.$relationJoin$conditionType"] = $valueCondition;
									$query[$conditions_type][] = "$table.id = $relationTable.$inverse_join";
								}
							} else {
								$query['fields'][$relationTable] = array();
								$query[$conditions_type]["$relationTable.$fieldName$conditionType"] = $valueCondition;
								$query[$conditions_type][] = "$table.id = $relationTable.$inverse_join";
								$query[$conditions_type][] = "$relationTable.id = $table.$relationJoin";
							}

							//Add field related_width_*
							if ($keyCondition == $conditionToPrev) {
								$query['fields'][$table][$relationJoin] = 'related_with_'.$prevTable;
							}

							//Unique value
							$this->result_limits[$arrayResult]['start'] = 0;
							$this->result_limits[$arrayResult]['limit'] = 1;
							break;

						default:
							$debug->db(__('There is not relations between the tables "%s" and "%s"', $table, $relationTable));

							break;
					}
				}
			}
		}

		//Limit
		if ($data['limit']) {
			if (is_array($data['limit'])) {
				$this->result_limits[$arrayResult]['start'] = intval($data['limit'][0]);
				$this->result_limits[$arrayResult]['limit'] = intval($data['limit'][1]);
			} else {
				$l = explode(',', $data['limit'], 2);

				if ($l[1]) {
					$this->result_limits[$arrayResult]['start'] = intval($l[0]);
					$this->result_limits[$arrayResult]['limit'] = intval($l[1]);
				} else {
					$this->result_limits[$arrayResult]['start'] = 0;
					$this->result_limits[$arrayResult]['limit'] = intval($l[0]);
				}
			}

			//If its the first table, define the limit
			if (!$prevTable) {
				$query['limit'] = array($this->result_limits[$arrayResult]['start'], $this->result_limits[$arrayResult]['limit']);
			}
		}

		//Order
		if (isset($data['sortCommands'])) {
			$query['sort'] = $data['sortCommands'];
		} else if ($data['sort']) {
			$query['sort'] = $this->sort($table, $data['sort']);
		}

		if ($data['group']) {
			$query['group'] = $data['group'];
		}

		//Fields commands
		$query['fieldsCommands'] = (array)$data['fieldsCommands'];

		//Make selection and save the result
		$this->result[$arrayResult] = parent::select($query);

		//Execute makeSelect recursively
		foreach((array)$data['addTables'] as $newData) {
			$newTable = $newData['table'];

			if (!$newTable['table']) {
				$debug->db(__('Table is not defined in the addTables selection'));
				continue;
			}

			$this->makeSelect($newTable, $newData, $table, $arrayResult);
		}
	}

	private function saveSelect ($table, $data, $prevTable = '', $arrayResult = '') {
		global $debug;

		$arrayResult = $prevTable.'.'.$table.'.'.$data['relation_name'];
		$result = array();
		$subResult = array();
		$id_relate = 'related_with_'.$table;

		//Get data from subtables
		foreach ((array)$data['addTables'] as $newData) {
			$newTable = $newData['table'];
			$newName = $newData['relation_name'] ? $newData['relation_name'] : $newTable;
			$tmpArray = $this->saveSelect($newTable, $newData, $table, $arrayResult);

			//Group array to $id_relate
			foreach ($tmpArray as $row) {
				$subResult[$row[$id_relate]][$newName][] = $row;
			}
		}

		if (!is_array($this->result[$arrayResult]) || (count($this->result[$arrayResult]) == 0)) {
			return array();
		}

		//Add data to $table result
		foreach ($this->result[$arrayResult] as $n => $row) {
			foreach ($row as $field => $value) {
				$row[$field] = stripslashes($value);
			}

			$result[$n] = $row;

			//Add data to subtables (in case of not data, always return an array)
			$tmpArray = $subResult[$row['id']];

			foreach ((array)$data['addTables'] as $newData) {
				$newTable = $newData['table'];
				$newName = $newData['relation_name'] ? $newData['relation_name'] : $newTable;
				$subArrayResult = $table.'.'.$newData['table'].'.'.$newData['relation_name'];

				if ($tmpArray[$newName]) {
					$result[$n][$newName] = $tmpArray[$newName];

					//In case of unique value, return an asociative array
					if ($this->result_limits[$subArrayResult]['limit'] == 1) {
						$result[$n][$newName] = current($result[$n][$newName]);

					//In case of limits, make an array_slice
					} else if ($this->result_limits[$subArrayResult]) {
						$result[$n][$newName] = array_slice($result[$n][$newName], $this->result_limits[$subArrayResult]['start'], $this->result_limits[$subArrayResult]['limit']);
					}
				} else {
					$result[$n][$newName] = array();
				}
			}
		}

		return $result;
	}

	public function valueForm ($table, $data, $suffix = '') {
		global $config, $debug;

		if (!($table = $this->tableExists($table))) {
			$debug->data(__('This table do not exists'));
			return false;
		}
		
		if (is_int(key($data))) {
			$unique = false;
		} else {
			$data = array($data);
			$unique = true;
		}

		$result = array();

		foreach ($data as $key => $row) {
			$data_group = $this->groupFields($table, $row);

			$result[$key] = array();

			foreach ($data_group as $field => $dbfields) {
				if (is_object($config->tables[$table][$field])) {
					$value_form = $config->tables[$table][$field]->valueForm($dbfields);

					foreach ($value_form as $f => $v) {
						$result[$key][$f.$suffix] = $v;
					}
				} else {
					$result[$key][$field.$suffix] = $dbfields;
				}
			}
		}

		return $unique ? current($result) : $result;
	}


	private function sort ($table, $sort) {
		global $config, $debug;

		$string = '';

		if (strstr($sort, ',')) {
			$sorts = explode(',', $sort);

			foreach ($sorts as $each) {
				$string .= $this->sort($table, $each);
			}
		} else {
			list($field, $mode) = explode(' ', trim($sort));

			$mode = ($mode == 'DESC')?'DESC':'ASC';

			if ($this->fieldExists($table, $field)) {
				$string = $table.'.'.$field.' '.$mode;
			} else if ($this->fieldExists($table, $field, true)) {
				$string = $table.'.'.key($config->tables[$table][$field]->settings).' '.$mode;
			}
		}

		return $string;
	}
}
?>
