<?php
/**
* phpCan v.1
*
* $Id: class_debug.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Debug {
	private $enable;
	private $errors = array();
	private $time;
	private $time_pause = 0;
	private $time_store = array();
	private $e_counter = 0;

	public function __construct () {
		global $config;

		$this->logs = $config->paths['logs'];
	}
	
	public function start () {
		global $config;

		$this->enable = $config->base['debug'];

		if ($this->enable) {
			$this->time = microtime(true);
		}
	}

	public function action ($message = '') {
		return $this->save('action', debug_backtrace(), $message);
	}

	public function data ($message = '') {
		return $this->save('data', debug_backtrace(), $message);
	}

	public function vars ($message = '') {
		return $this->save('vars', debug_backtrace(), $message);
	}

	public function event ($message = '') {
		return $this->save('event', debug_backtrace(), $message);
	}

	public function template ($message = '') {
		return $this->save('template', debug_backtrace(), $message);
	}

	public function format ($message = '') {
		return $this->save('format', debug_backtrace(), $message);
	}

	public function db ($message = '') {
		return $this->save('db', debug_backtrace(), $message);
	}

	private function save ($where, $info, $message) {
		$this->errors[] = array(
			'where' => $where,
			'info' => $info,
			'message' => $message
		);

		$log = $this->logs.$where.'.php';
		$exists = is_file($this->logs.$where.'.php');

		if ($exists && !is_writable($this->logs.$where.'.php')) {
			return false;
		} else if (!$exists && !is_writable($this->logs)) {
			return false;
		}

		if ($exists) {
			$fp = fopen($log,'a+');
		} else {
			$fp = fopen($log,'w');
			$ini = '<?php';
		}

		if (!$fp) {
			return false;
		}

		flock($fp, LOCK_EX);
		fwrite($fp, $ini."\n//".(is_array($info) ? serialize($info) : $info));
		flock($fp, LOCK_UN);
		fclose($fp);

		return false;
	}

	public function showErrors () {
		global $data;

		if (!$this->enable) {
			return false;
		}

		if (!$this->errors && !$data->error) {
			return false;
		}

		include_once($config->path['includes'].'utils/debug_styles.php');

		echo '<hr />';

		//Data errors
		foreach ($data->error as $table => $val) {
			foreach ($val as $field => $error) {
				echo '<div class="phpcan_debug_show"><p class="msg">';

				foreach ($error as $dbfield => $dberror) {
					echo '<strong>'.$table.' -> '.$field.' -> '.$dbfield.':</strong>'.$dberror.'<br />';
				}

				echo '</p></div>';
			}
		}

		//Debug errors
		foreach ($this->errors as $error) {
			$error['info'] = array_reverse($error['info']);

			echo '<div class="phpcan_debug_show">';

			if ($error['message']) {
				echo '<p class="msg">'.$error['message'].'</p>';
			}

			echo '<ol>';

			foreach ($error['info'] as $v) {
				switch ($v['function']) {
					case 'include':
					case 'include_once':
					case 'require':
					case 'require_once':
					continue 2;
					break;

					default:
					echo '<li><dl>';
					echo '<dt>File: </dt>';
					echo '<dd>'.$v['file'].' <strong>(line '.$v['line'].')</strong></dd>';
					echo '<dt>Function: </dt>';
					echo '<dd><strong>'.$v['function'].'</strong> (';
					echo '<code>';
					if (is_array($v['args'][0])) {
						print_r($v['args'][0]);
					} else {
						echo '"'.$v['args'][0].'"';
					}
					echo '</code>';
					echo ')</dd>';
					echo '</dl></li>';
				}
			}

			echo '</ol>'
				.'</div>';
		}
	}
	
	/**
	* public function showData ([string $filter])
	*
	* Return none
	*/
	public function showData ($filter = '') {
		global $data, $config, $html;

		if (!$this->enable) {
			return false;
		}

		if (!$data->data) {
			return false;
		}

		$variables = array();

		if ($filter) {
			$filter = explode(',', $filter);

			foreach ($filter as $v) {
				$v = trim($v);

				if (in_array($v, $data->data)) {
					$variables[] = $v;
				}
			}
		} else {
			$variables = $data->data;
		}

		include_once($config->path['includes'].'utils/debug_styles.php');

		echo '<hr />';
		echo '<div class="phpcan_debug_show">';

		echo '<ul class="phpcan_debug_menu">';
		foreach ($variables as $key) {
			echo '<li>'.$html->a($key, '#phpcan_debug_data_'.$key).'</li>';
		}
		echo '</ul>';

		foreach ($variables as $key) {
			global $$key;
			$value = $$key;

			echo '<h1 id="phpcan_debug_data_'.$key.'">$'.$key.'</h1>';
			echo '<code>';
			print_r($value);
			echo '</code>';
		}

		echo '</div>';
	}

	/**
	* public function e ()
	*
	* Return none
	*/
	public function e ($var, $title = '', $level = 0) {
		if (!$this->enable) {
			return false;
		}

		include_once($config->path['includes'].'utils/debug_styles.php');

		$where = debug_backtrace();
		$where = $where[$level];

		echo '<hr />';
		echo '<pre class="phpcan_debug_e">';

		if ($title) {
			echo '<h1>'.$title.' ('.gettype($var).')</h1>'."\n";
		} else {
			echo '<h1>'.gettype($var).'</h1>'."\n";
		}

		echo '<span class="subtitle">'.$this->e_counter.' / '.$where['file'].' <strong>(line '.$where['line'].')</strong></span>'."\n";

		$this->e_counter++;

		if (is_string($var)) {
			echo $var;
		} else if (is_bool($var)) {
			echo $var ? 'true' : 'false';
		} else {
			print_r($var);
		}

		echo '</pre>';
	}
	
	/**
	* public function resetTiming()
	*/
	public function resetTiming() {
		$this->time_store = array();
		$this->time = microtime(true);
	}

	/**
	* public function pauseTiming()
	*/
	public function pauseTiming() {
		$this->time_pause = microtime(true);
	}

	/**
	* public function playTiming()
	*/
	public function playTiming() {
		if ($this->time_pause > 0) {
			$this->time += (microtime(true) - $this->time_pause);
			$this->time_pause = 0;
		}
	}

	/**
	* public function timingMark([string $text])
	*/
	public function markTiming($text = '') {
		$text = $text ? $text : 'Show timing';
		$where = current(debug_backtrace());

		$this->playTiming();

		return $this->time_store[] = array(
			'text' => $text,
			'time' => microtime(true) - $this->time,
			'where' => $where['file'].' <strong>(line '.$where['line'].')</strong>'
		);
	}

	/**
	* public function timingShow([string $text])
	*/
	public function showTiming($text = '') {
		$show = $this->markTiming($text);

		$this->e($show['time'], $show['text'], 1);
	}

	/**
	* public function timingShowAll([string $text])
	*/
	public function showTimingAll($text = '') {
		$this->markTiming($text);

		$this->e($this->time_store, 'Show timing all', 1);
	}
}
?>
