<?php
/**
* phpCan v.1
*
* $Id: class_html.php 356 2009-08-06 16:45:37Z oscar $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Html {
	private $once = array();
	private $counter = 1;
	public $tabindex = 1;

	/**
	* function a ([string $text], [string $href], [array $params])
	*
	* Return string
	*/
	public function a ($text = array(), $href = array(), $params = array()) {
		if (is_array($text)) {
			$params += $text;
			$text = $params['text'];
			unset($params['text']);

		} else if (!is_array($href)) {
			$href = array('href' => $href);
		}

		$params = array_merge($params, $href);

		//Params optimization
		if ($params['href']) {
			$params['href'] = str_replace('&amp;', '&', $params['href']);
			$params['href'] = str_replace('&', '&amp;', $params['href']);
		} else if ($params['id']) {
			$params['href'] = '#'.$params['id'];
		} else {
			$params['href'] = path();
		}

		if ($params['target'] == '_blank') {
			unset($params['target']);
			$params['onclick'] .= 'window.open(this.href); return false;';
		}

		return '<a'.$this->params($params).'>'.$text.'</a>';
	}
	
	/**
	* function aList (array $array_data, string $text, string $href, [array $params], [string $base_href], [string $separator])
	*
	* Return string/array
	*/
	public function aList ($array_data, $text, $href, $params = array(), $base_href = '', $separator = '') {
		$links = array();

		if (is_string($params)) {
			$separator = $base_href;
			$base_href = $params;
			$params = array();
		}

		foreach ((array)$array_data as $value) {
			$links[] = $this->a($value[$text], $base_href.$value[$href], $params);
		}

		if ($separator) {
			$links = implode($separator, $links);
		}

		return $links;
	}

	/**
	* function img ([string $src], [string $alt], [array $params])
	*
	* Return string
	*/
	public function img ($src = array(), $alt = array(), $params = array()) {
		if (is_array($src)) {
			$params += $src;
		} else {
			$params['src'] = $src;
			
			if (is_array($alt)) {
				$params += $alt;
			} else {
				$params['alt'] = $alt;
			}
		}

		if (!$params['src']) {
			return '';
		}

		//Transform params
		if ($params['transform']) {
			$transform = array(
				'src' => $params['src'],
				'transform' => $params['transform']
			);
			$params['src'] = path('img2.php').get($transform, false);

			unset ($params['transform']);
		}

		//Params optimization
		if (!$params['alt']) {
			$params['alt'] = '';
		}
		$params['src'] = str_replace('&amp;', '&', $params['src']);
		$params['src'] = str_replace('&', '&amp;', $params['src']);

		return '<img'.$this->params($params).' />';
	}

	/**
	* function formInputParams ([string $value], [string $name], [array $params], [array $label])
	*
	* Return array
	*/
	private function formInputParams ($value, $name, $params, $label = false) {
		if (!is_array($value)) {
			$value = array('value' => $value);

			if (!is_array($name)) {
				$name = array('name' => $name);
			} else {
				$label = $params;
				$params = array();
			}
		} else {
			$label = $name;
			$name = array();
		}

		if (!is_array($params)) {
			$label = $params;
			$params = array();
		}

		$params = array_merge($params, $value, $name);

		//Default value
		if ($params['defaultValue']) {
			if (!$params['value']) {
				$params['value'] = $params['defaultValue'];
			}
		} else {
			$params['defaultValue'] = $params['value'];
		}

		//Variables
		if ($params['variable']) {
			global $vars;

			if (isset($vars->var[$params['variable']])) {
				$params['value'] = $vars->var[$params['variable']];
			}
			$params['name'] = $params['variable'];
		}
		unset ($params['variable']);
		
		//Autoclear
		if ($params['autoclear']) {
			$params['onfocus'] .= "if (this.value == '".$params['defaultValue']."') { this.value = '' };";
			$params['onblur'] .= "if (this.value == '') { this.value = '".$params['value']."' };";
		}
		unset ($params['autoclear'], $params['defaultValue']);

		//Autosubmit
		if ($params['autosubmit']) {
			$params['onchange'] .= 'this.form.submit();';
		}
		unset ($params['autosubmit']);

		//Overwrite_control
		if (array_key_exists('overwrite_control', $params)) {
			$overwrite_control = $this->formHidden(array(
				'value' => $params['overwrite_control'],
				'name' => $params['overwrite_control_suffix'] ? $params['name'].$params['overwrite_control_suffix'] : $params['name'].'_default'
			));
		}
		unset ($params['overwrite_control'], $params['overwrite_control_suffix']);

		//Tabindex
		if (!$params['tabindex']) {
			$params['tabindex'] = $this->tabindex;
		} else {
			$this->tabindex = $params['tabindex'];
		}

		$this->tabindex++;

		//Errors
		$error = '';
		if ($params['error']) {
			$class = 'error';
			$class .= $params['error_class'] ? ' '.$params['error_class'] : '';

			if ($params['id']) {
				$for = $params['id'];
			} else if ($label['for']) {
				$for = $params['id'] = $label['for'];
			} else {
				$for = $params['id'] = $params['name'];
			}
			$error = '<label for="'.$for.'" class="'.$class.'">'.$params['error'].'</label>';
			$params['class'] .= $params['class'] ? ' error' : 'error';
		}
		unset ($params['error'], $params['error_class']);

		return array('params' => $params, 'label' => $label, 'error' => $error, 'overwrite_control' => $overwrite_control);
	}

	/**
	* function formLabel ([array $params], [array $label])
	*
	* Return array
	*/
	private function formLabel ($params) {
		if (!$params['label']) {
			return $params;
		}
		
		if (is_array($params['label'])) {
			$labelText = $params['label']['text'] ? $params['label']['text'] : 'text param?';
			unset($params['label']['text']);
		} else {
			$labelText = $params['label'];
			$params['label'] = array();
		}

		if ($params['params']['id']) {
			$params['label']['for'] = $params['params']['id'];
		} else if ($params['label']['for']) {
			$params['params']['id'] = $params['label']['for'];
		} else {
			$params['params']['id'] = $params['label']['for'] = $params['params']['name'];
		}

		$position = $params['label']['position'];
		unset ($params['label']['position']);

		$label = '<label'.$this->params($params['label']).'>'.$labelText.'</label>';
		unset ($params['label']);

		if ($position == 'after') {
			$params['label_after'] = ' '.$label;
		} else {
			$params['label_before'] = $label.' ';
		}

		return $params;
	}

	/**
	* function formSubmit ([string $value], [string $name], [array $params])
	*
	* Return string
	*/
	public function formSubmit ($value = '', $name = '', $params = array()) {
		$params = $this->formInputParams($value, $name, $params);
		$params['params']['type'] = 'submit';
		$params['params']['class'] .= $params['params']['class'] ? ' submit' : 'submit';

		if (!$params['params']['name']) {
			unset($params['params']['name']);
		}

		return '<input'.$this->params($params['params']).' />'.$params['error'].$params['overwrite_control'];
	}

	/**
	* function formHidden ([string $value], [string $name], [array $params])
	*
	* Return string
	*/
	public function formHidden ($value = '', $name = '', $params = array()) {
		$params = $this->formInputParams($value, $name, $params);

		if (!$params['params']['name']) {
			return '';
		}

		unset ($params['params']['tabindex']);
		$this->tabindex--;

		$params['params']['type'] = 'hidden';

		return '<input'.$this->params($params['params']).' />'.$params['overwrite_control'];
	}

	/**
	* function formText ([string $value], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formText ($value = '', $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams($value, $name, $params, $label);
		$params['params']['type'] = 'text';
		$params['params']['class'] .= $params['params']['class'] ? ' text' : 'text';
		$params = $this->formLabel($params, $label);

		return $params['label_before'].'<input'.$this->params($params['params']).' />'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}
	
	/**
	* function formTextarea ([string $value], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formTextarea ($value = '', $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams($value, $name, $params, $label);
		$params = $this->formLabel($params, $label);
		$text = $params['params']['value'];
		unset($params['params']['value']);

		return $params['label_before'].'<textarea'.$this->params($params['params']).'>'.$text.'</textarea>'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}
	
	/**
	* function formPassword ([string $value], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formPassword ($value = '', $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams($value, $name, $params, $label);
		$params['params']['type'] = 'password';
		$params['params']['class'] .= $params['params']['class'] ? ' password' : 'password';
		$params = $this->formLabel($params, $label);

		return $params['label_before'].'<input'.$this->params($params['params']).' />'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}

	/**
	* function formFile ([string $value], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formFile ($name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams(NULL, $name, $params, $label);
		$params['params']['type'] = 'file';
		$params['params']['class'] .= $params['params']['class'] ? ' file' : 'file';
		unset($params['params']['value']);
		$params = $this->formLabel($params, $label);

		return $params['label_before'].'<input'.$this->params($params['params']).' />'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}
	
	/**
	* function formCheckbox ([string $value], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formCheckbox ($value = '', $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams($value, $name, $params, $label);
		$params['params']['type'] = 'checkbox';
		$params['params']['class'] .= $params['params']['class'] ? ' checkbox' : 'checkbox';

		if ($params['params']['checked']) {
			$params['params']['checked'] = 'checked';
		} else {
			unset($params['params']['checked']);
		}

		if (!$params['params']['tabindex']) {
			$params['params']['tabindex'] = $this->tabindex;
		} else {
			$this->tabindex = $params['params']['tabindex'];
		}

		$this->tabindex++;

		$params = $this->formLabel($params, $label);
		$hidden = '';

		if (isset($params['params']['force'])) {
			unset($params['params']['force']);
		} else {
			$hidden = $this->formHidden(0, $params['params']['name']);
		}

		return $hidden.$params['label_before'].'<input'.$this->params($params['params']).' />'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}
	
	/**
	* function formSelect ([array $values], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formSelect ($values = array(), $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams(NULL, $name, $params, $label);
		$params = $this->formLabel($params, $label);

		$options = '';
		$selected = $params['params']['value'];
		unset($params['params']['value']);

		if (isset($params['params']['first_option'])) {
			$options .= '<option value="">'.$params['params']['first_option'].'</option>'."\n";

			unset($params['params']['first_option']);
		}

		//Complex array
		if (($option_value = $params['params']['option_value']) && ($option_text = $params['params']['option_text'])) {
			$option_selected = $params['params']['option_selected'];
			unset($params['params']['option_value'], $params['params']['option_text'], $params['params']['option_selected']);

			foreach ((array)$values as $option) {
				$option_params = $this->params(array(
					'value' => $option[$option_value],
				'selected' => (($selected && $option[$option_value] == $selected) || ($option_selected && $option[$option_selected])) ? 'selected' : NULL
			));

			$options .= '<option'.$option_params.'>'.$option[$option_text].'</option>'."\n";
			}

		//Simple array (key->value)
		} else {
			foreach ((array)$values as $value => $option) {
				$option_params = $this->params(array(
					'value' => $value,
					'selected' => $value == $selected ? 'selected' : NULL
				));

				$options .= '<option'.$option_params.'>'.$option.'</option>'."\n";
			}
		}

		return $params['label_before'].'<select'.$this->params($params['params']).'>'."\n".$options."\n".'</select>'.$params['label_after']."\n".$params['error'].$params['overwrite_control'];
	}


	/**
	* function formRadio ([array $values], [string $name], [array $params], [array $label])
	*
	* Return string
	*/
	public function formRadio ($values = array(), $name = '', $params = array(), $label = array()) {
		$params = $this->formInputParams(NULL, $name, $params, $label);

		$params['params']['type'] = 'radio';
		$result = '';
		$selected = $params['params']['value'];
		unset($params['params']['value']);

		//Complex array
		if (($option_value = $params['params']['option_value']) && ($option_text = $params['params']['option_text'])) {
			$option_selected = $params['params']['option_selected'];
			unset($params['params']['option_value'], $params['params']['option_text'], $params['params']['option_selected']);

			$n = 1;
			foreach ((array)$values as $option) {
				$params['params']['value'] = $option[$option_value];
				$params['params']['id'] = $params['params']['name'].'_'.$n++;
				$params['params']['checked'] = (($selected && $option[$option_value] == $selected) || ($option_selected && $option[$option_selected])) ? 'selected' : NULL;
				$params['label']['text'] = $option[$option_text];

				$labelParams = $this->formLabel($params, $label);

				$result .= $labelParams['label_before'].'<input'.$this->params($params['params']).' />'.$labelParams['label_after']."\n";
			}

		//Simple array (key->value)
		} else {
			$n = 1;
			foreach ((array)$values as $value => $option) {
				$params['params']['value'] = $value;
				$params['params']['id'] = $params['params']['name'].'_'.$n++;
				$params['params']['checked'] = $value == $selected ? 'selected' : NULL;
				$params['label']['text'] = $option;

				$labelParams = $this->formLabel($params, $label);

				$result .= $labelParams['label_before'].'<input'.$this->params($params['params']).' />'.$labelParams['label_after']."\n";
			}
		}

		return $result."\n".$params['error'].$params['overwrite_control'];
	}


	/**
	* public jsLink (string $file, [string $path], [array $params])
	*
	* Create the code to import a javascript file
	*
	* return string
	*/
	public function jsLink ($file, $path = false, $params = array()) {
		if (is_array($file)) {
			$params = $file;
		} else {
			if ($path === false) {
				global $config, $vars;
				$path = $vars->isAdmin() ? $config->paths['www-admin-templates'] : $config->paths['www-scene-templates'];
				$path .= 'js/';
			}

			$params['src'] = $path.$file;
		}

		if (!$this->once($params['src'])) {
			return false;
		}

		$params['type'] = 'text/javascript';

		return '<script'.$this->params($params).'></script>';
	}


	/**
	* public cssLink (string $file, [string $path])
	*
	* Create the code to import a css file
	*
	* return string
	*/
	public function cssLink ($file, $path = false, $params = array()) {
		if (is_array($file)) {
			$params = $file;
		} else {
			if ($path === false) {
				global $config, $vars;
				$path = $vars->isAdmin() ? $config->paths['www-admin-templates'] : $config->paths['www-scene-templates'];
				$path .= 'css/';
			}

			$params['href'] = $path.$file;
		}

		$params['type'] = 'text/css';
		$params['rel'] = 'stylesheet';

		return '<link'.$this->params($params).' />';
	}


	/**
	* public dinamicCssLink (string $file, [string $path])
	*
	* Create the code to import a css file by javascript
	*
	* return string
	*/
	public function dinamicCssLink ($file, $path = false) {
		if (!$this->once($path.$file)) {
			return false;
		}

		if ($path === false) {
			global $config, $vars;
			$path = $vars->isAdmin() ? $config->paths['www-admin-templates'] : $config->paths['www-scene-templates'];
			$path .= 'css/';
		}

		return '<script type="text/javascript">'
				.'var headID = document.getElementsByTagName("head")[0];'
				.'var cssNode = document.createElement("link");'
				.'cssNode.type = "text/css";'
				.'cssNode.rel = "stylesheet";'
				.'cssNode.href = "'.$path.$file.'";'
				.'headID.appendChild(cssNode);'
				.'</script>';
	}


	/**
	* function params (array $params)
	*
	* Return string
	*/
	public function params ($params) {
		if (!is_array($params)) {
			return '';
		}

		$txt = '';

		foreach ($params as $key => $value) {
			if (is_null($value)) {
				continue;
			}
			switch ($key) {
				case 'selected':
				case 'checked':
				case 'disabled':
				if ($value) {
					$value = $key;
				} else {
					continue 2;
				}
			}
			$txt .= ' '.$key.'="'.escapeForm($value).'"';
		}

		return $txt;
	}
	
	/**
	* function once (string $text)
	*
	* Return the same string once
	*
	* Return string
	*/
	public function once ($text) {
		$code = md5($text);

		if (!in_array($code, $this->once)) {
			$this->once[] = $code;
			return $text;
		} else {
			return '';
		}
		
	}
	
	/**
	* function each (string $text, int $number, [bool $increment])
	*
	* Return the string in multiples of $number
	*
	* Return string
	*/
	public function each ($text, $number, $increment = true) {
		$txt = '';

		if ($this->counter > 1 && ($this->counter%$number) == 0) {
			$txt = $text;
		}
		if ($increment) {
			$this->counter++;
		}

		return $txt;
	}
	
	/**
	* function eachStart ()
	*
	* Return none
	*/
	public function eachStart () {
		$this->counter = 1;
	}

	/**
	* function textCutter (string $text, [int $limit], [$string $end])
	*
	* Return string
	*/
	function textCutter ($text, $limit = 140, $end = '...') {

		if (strlen($text) <= $limit) {
			return $text;
		}

		$text = substr($text, 0, $limit + 1);
		$space = strlen(strrchr($text, ' '));
		if ($space) {
			$text = substr($text, 0, -$space);
		}

		return $text.$end;
	}


	/**
	* function loremIpsum ([int $size])
	*
	* Return string
	*/
	function loremIpsum ($size = 200) {
		$text = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc nec mollis sapien. Sed eget erat quis turpis semper tincidunt. Suspendisse aliquet, leo quis accumsan dignissim, risus diam condimentum purus, eu pharetra quam justo ut turpis. Phasellus vitae tellus neque. Sed tellus justo, pretium eu viverra a, ornare sit amet sem. In placerat, nisi at sollicitudin volutpat, ante arcu molestie lacus, a lacinia nisi urna a risus. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam in metus vitae neque commodo vehicula. Donec ut dignissim quam. Nullam vestibulum orci sit amet risus congue ultricies. Cras ultrices, lorem nec tincidunt ornare, massa massa ultrices dui, eget ornare dui nulla id sem. Fusce urna est, dapibus nec porta sit amet, tempor sagittis velit. Ut sem lacus, congue sit amet mollis in, molestie et libero. Integer at neque quis massa laoreet faucibus nec non libero. Nulla facilisi. Vestibulum ultrices adipiscing ornare. Aliquam quis est in tortor vehicula egestas. Vivamus eleifend, urna id iaculis tempor, enim felis lacinia urna, nec sagittis magna purus vitae ante. Curabitur non tellus turpis. Fusce sit amet elementum urna. Vivamus egestas pretium nibh cursus euismod. Nam at nibh at nisl aliquet vulputate id vel urna. Pellentesque accumsan ipsum in risus varius et aliquet augue mattis. Nulla lorem sem, porttitor eget varius ac, aliquet ac enim. Duis nunc ipsum, congue vitae congue sit amet, lacinia et neque. Aliquam erat volutpat. Nunc accumsan, felis sit amet vestibulum consectetur, nulla dolor pharetra justo, eu sagittis nisl metus ac neque. Maecenas ornare placerat volutpat. Cras nec nisl ligula. Ut vel nisi nec sapien tempor lobortis. In hac habitasse platea dictumst. Vivamus ut enim sit amet odio consequat mattis id lacinia odio. Integer vitae nunc vel erat consectetur molestie. Pellentesque aliquet, tortor at eleifend gravida, est tortor sodales justo, eget ullamcorper sapien est sit amet ligula. Sed auctor, lorem eget ultricies lobortis, diam erat ultricies nisi, vitae ultricies est nulla in leo. Nulla facilisi. Donec in magna eu lorem congue ullamcorper. Duis luctus nibh id velit rhoncus sodales. Pellentesque arcu libero, molestie eu gravida in, pretium vitae augue. Donec quis luctus mauris. Maecenas metus erat, vehicula id congue eget, semper in ligula. Quisque a elit a metus suscipit aliquam. Nam aliquet, diam in molestie mattis, leo arcu sagittis ante, eget venenatis nibh odio non urna. Nulla facilisi. Sed id nunc nec leo blandit interdum ac ac ipsum. Sed tempus, felis in tincidunt porta, sem neque bibendum nisl, vitae ullamcorper leo arcu sed odio. Aliquam ultrices nisi suscipit elit sollicitudin ut pharetra nisl volutpat. Integer purus nunc, venenatis id fringilla sit amet, sagittis vitae sem. Nunc rutrum tortor sit amet leo dictum imperdiet. Nullam gravida eleifend velit elementum convallis. Sed ac nunc ac ligula vestibulum posuere. Nunc varius, ligula nec sollicitudin pellentesque, mauris velit congue ante, eget sodales lacus nunc non urna. Phasellus nisi sapien, sollicitudin ac rutrum at, auctor ut odio. Aliquam vulputate feugiat convallis. In pharetra lorem non neque euismod convallis. Donec tincidunt nisl quis eros tristique et hendrerit sem fringilla. Fusce mauris justo, porttitor et vehicula in, dictum eu nulla. Proin luctus porttitor massa vitae commodo. Proin dictum molestie purus, vel rhoncus lacus facilisis non. Quisque at faucibus dolor. Quisque malesuada, arcu nec ultricies ultrices, justo est mattis orci, eu facilisis orci ante imperdiet elit. Suspendisse potenti. Maecenas vehicula, augue sed blandit elementum, nibh augue tincidunt velit, a ornare mauris sapien sit amet velit. Donec in urna orci. Donec quam tortor, sollicitudin quis fringilla sit amet, imperdiet in sem. Sed ligula lectus, euismod vitae adipiscing mollis, rutrum nec arcu. Aliquam aliquam laoreet nisi, vel porta ligula pellentesque eget. Morbi at tortor mauris, facilisis interdum augue. Etiam placerat rutrum gravida. Duis nibh diam, venenatis eget convallis eu, laoreet interdum neque. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos.';

		while ($size > strlen($text)) {
			$text .= $text;
		}

		$text = $this->textCutter($text, $size, '');
		$lastChar = substr($text, -1);
		if ($lastChar == '.' || $lastChar == ',') {
			$text = substr($text, 0, -1);
		}

		return $text.'.';
	}
}
?>
