<?php
/**
* phpCan v.1
*
* $Id$
*
* 2009 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();


if (extension_loaded('imagick')) {
	$library = 'imagick';
} else if (extension_loaded('gd')) {
	$library = 'gd';
} else {
	$debug->e(__('No image library is loaded'));
	die();
}

global $config;

require_once($config->paths['includes'].'class_'.$library.'.php');

class Image extends Image_library {
	private $cache = false;

	/**
	* public function transform ([string $operations], [boolean $cache])
	*
	* Execute a list of operations
	*/
	public function transform ($operations = '') {
		if (!$operations) {
			return $this;
		}

		if ($this->loadCache($operations)) {
			return $this;
		}

		$array_operations = explode('|', $operations);

		foreach ($array_operations as $operation) {
			$params = explode(',', $operation);

			if (empty($params[0])) {
				array_shift($params);
			}

			$function = $params[0];

			switch ($function) {
				case 'flip':
				case 'flop':
				$this->$function();
				break;

				case 'convert':
				$this->$function($params[1]);
				break;

				case 'resize':
				case 'zoomCrop':
				case 'rotate':
				$this->$function($params[1], intval($params[2]));
				break;

				case 'merge':
				$this->$function($params[1], $params[2], $params[3]);
				break;

				case 'crop':
				$this->$function($params[1], $params[2], $params[3], $params[4]);
				break;
			}
		}

		$this->saveCache($operations);

		return $this;
	}
	
	/**
	* public function get ([string $image])
	*
	* return the image object
	*/
	public function get ($image = '') {
		if ($image) {
			$this->load($image);
		}

		return $this->image;
	}


	/**
	* public function set ([object $image])
	*
	* Add an image to the class
	*/
	public function set ($image) {
		$this->image = $image;

		return $this;
	}


	/**
	* protected function position (int/string $position, int $size, int $canvas)
	*
	* Return integer
	*/
	protected function position ($position, $size, $canvas) {
		if (is_int($position)) {
			return $position;
		}

		switch ($position) {
			case 'top':
			case 'left':
				$position = 0;
				break;

			case 'middle':
			case 'center':
				$position = ($canvas/2) - ($size/2);
				break;

			case 'right':
			case 'bottom':
				$position = $canvas - $size;
				break;

			default:
				$position = 0;
		}

		return $position;
	}


	/**
	* private function saveCache (string $operations)
	*
	* Return boolean
	*/
	private function saveCache ($operations) {
		if (!$this->cache) {
			return false;
		}

		$cache_file = $this->cache.md5($this->info['file'].$operations);

		$this->save($cache_file);

		return true;
	}


	/**
	* private function loadCache (string $operations)
	*
	* Return boolean
	*/
	private function loadCache ($operations) {
		if (!$this->cache) {
			return false;
		}

		$cache_file = $this->cache.md5($this->info['file'].$operations);

		if (!is_file($cache_file)) {
			return false;
		}

		$this->load($cache_file);

		return true;
	}


	/**
	* public function setCache (string $folder)
	*/
	public function setCache ($folder) {
		if (!$folder) {
			$this->cache = false;
		} else if (is_dir($folder) && is_writable($folder)) {
			$this->cache = $folder;
		} else if (mkdir($folder)) {
			$this->cache = $folder;
		} else {
			$this->cache = false;
		}
	}
}
?>