<?php
/**
* phpCan v.1
*
* $Id: class_mysql.php 343 2009-07-22 20:12:43Z oscar $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

abstract class DataBase {
	private $res;

	protected $conn = false;

	public $query;
	public $queryRegister = array();
	public $error;
	public $data;
	public $last_id;

	abstract protected function setTables ();
	abstract protected function setRelations ();
	abstract protected function multipleQueries ($q);

	/**
	* function connect (void)
	*
	* Start an MySQL connection
	*/
	public function connect () {
		if ($this->conn) {
			return true;
		}

		global $config, $debug;

		if ($config->db['persistent']) {
			$this->conn = @mysql_pconnect($config->db['host'], $config->db['user'], $config->db['password']);
		} else {
			$this->conn = @mysql_connect($config->db['host'], $config->db['user'], $config->db['password']);
		}

		if (!$this->conn) {
			$debug->db(mysql_error());
			include ($config->paths['includes'].'utils/no-connection_template.php');

			exit;
		}

		if (!@mysql_select_db($config->db['database'], $this->conn)) {
			$debug->db(mysql_error());
			include ($config->paths['includes'].'utils/no-connection_template.php');

			exit;
		}

		mysql_query("SET NAMES 'utf8';");

		return true;
	}

	/**
	* function loadDifferentSchema (void)
	*
	* Check differences between config tables definitions
	* and the actual database schema
	*
	* return string
	*/
	public function loadDifferentSchema () {
		global $config, $debug;

		//Get real table definitions
		$real_tables = array();
		$real_indexes = array();

		$tmp_tables = $this->loadQuery('SHOW TABLES;');

		foreach ($tmp_tables as $table) {
			$table = current($table);

			//Get indexes
			$indexes = $this->loadQuery('SHOW INDEX FROM `'.$table.'`;');

			foreach ($indexes as $index) {
				if ($index['Non_unique']) {
					$type = 'index';
				} else if ($index['Key_name'] == 'PRIMARY') {
					$type = 'key';
				} else {
					$type = 'unique';
				}

				$real_indexes[$table][$index['Column_name']][$type] = $index['Key_name'];
			}

			//Get fields
			$describe = $this->loadQuery('DESCRIBE `'.$table.'`;');

			foreach ($describe as $fields) {
				$type = $fields['Type'];
				$field = $fields['Field'];

				preg_match('#^(\w+)(\((.+)\))?( unsigned)?$#', $type, $t);

				if (!strstr($t[3], "'")) { //For enum values
					if (strstr($t[3], ',')) {
						list($integers, $decimals) = explode(',', $t[3]);

						$t[3] = intval($integers).','.intval($decimals);
					} else {
						$t[3] = intval($t[3]);
					}
				}

				$real_tables[$table][$field] = array(
					'type' => $t[1] == 'int' ? 'integer' : $t[1],
					'length_max' => $t[3],
					'unsigned' => $t[4] ? true : false,
					'null' => $fields['Null'] == 'NO' ? false : true,
					//'key' => $index_fields[$field]['key'],
					//'index' => $index_fields[$field]['index'],
					//'unique' => $index_fields[$field]['unique'],
					'incremental' => strstr($fields['Extra'], 'auto_increment') ? true : false,
					'default' => $fields['Default'],
				);

				unset($t);
			}
		}

		//Get virtual table definitions
		$virtual_tables = array();

		foreach ($config->fields as $table => $fields) {
			foreach ($fields as $field => $format) {
				$settings = $config->tables[$table][$format]->settings[$field];

				//Get indexes
				$virtual_indexes[$table][$field] = array(
					'key' => $settings['db_key'] ? 'PRIMARY' : '',
					'index' => $settings['db_index'],
					'unique' => $settings['db_unique'],
				);

				if ($settings['db_type'] == 'enum') {
					$settings['db_length_max'] = "'".implode("','", $settings['db_values'])."'";
				} else {
					if (strstr($settings['db_length_max'], ',')) {
						list($integers, $decimals) = explode(',', $settings['db_length_max']);

						$settings['db_length_max'] = intval($integers).','.intval($decimals);
					} else {
						$settings['db_length_max'] = intval($settings['db_length_max']);
					}
				}

				//Get fields
				$virtual_tables[$table][$field] = array(
					'type' => $settings['db_type'],
					'length_max' => $settings['db_length_max'],
					'unsigned' => $settings['db_unsigned'],
					'null' => $settings['db_null'],
					//'key' => $settings['db_key'],
					//'index' => $settings['db_index'],
					//'unique' => $settings['db_unique'],
					'incremental' => $settings['db_incremental'],
					'default' => $settings['db_default'],
				);
			}
		}

		//Add relations fields to table definitions
		foreach ($config->relations as $table1 => $value) {
			if (!$virtual_tables[$table1]) {
				continue;
			}

			foreach ($value as $table2 => $relations) {
				if (!$virtual_tables[$table2]) {
					continue;
				}

				foreach ($relations as $settings) {
					switch ($settings['mode']) {
						case '1 1':
							$virtual_tables[$table1][$settings['join']] = array(
								'type' => 'integer',
								'unsigned' => true,
								'length_max' => 10
							);
							break;

						case '1 x':
							$virtual_tables[$table2][$settings['join']] = array(
								'type' => 'integer',
								'unsigned' => true,
								'length_max' => 10
							);
							break;

						case 'x x':
							if ($settings['relation_table'] != $this->phpcan_tables['relations']) {
								$virtual_tables[$settings['relation_table']]['id1'] = $virtual_tables[$settings['relation_table']]['id2'] = array(
									'type' => 'integer',
									'unsigned' => true,
									'length_max' => 10,
								);

								$virtual_tables[$settings['relation_table']]['tables'] = array(
									'type' => 'varchar',
									'length_max' => 100
								);
							}
							break;
					}
				}
			}
		}

		//Remove id field from relations table
		unset($virtual_tables[$this->phpcan_tables['relations']]['id']);

		//Init the query and create an array width all tables
		$query = array();

		$all_tables = array_merge(array_keys($real_tables), array_keys($virtual_tables));
		$all_tables = array_unique($all_tables);

		//Compare real_tables and virtual_tables fields and create the query
		foreach ($all_tables as $table) {

			//Add table
			if (!$real_tables[$table]) {
				$query_fields = array();

				//Insert fields and keys
				foreach ($virtual_tables[$table] as $field => $settings) {
					$query_fields[] =  '`'.$field.'` '.$this->fieldSchema($settings);
				}

				$query[] = 'CREATE TABLE `'.$table.'` ('.implode(', ', $query_fields).') ENGINE=MyISAM DEFAULT CHARSET=utf8;';

				continue;
			}

			//Drop table
			if (!$virtual_tables[$table]) {
				$query[] = 'DROP TABLE `'.$table.'`;';

				continue;
			}

			$all_fields = array_keys($virtual_tables[$table]) + array_keys($real_tables[$table]);
			$all_fields = array_unique($all_fields);

			foreach ($all_fields as $field) {

				//Add field
				if (!$real_tables[$table][$field]) {
					$query[] = 'ALTER TABLE `'.$table.'` ADD `'.$field.'` '.$this->fieldSchema($virtual_tables[$table][$field]).';';

					continue;
				}

				//Drop field
				if (!$virtual_tables[$table][$field]) {
					$query[] = 'ALTER TABLE `'.$table.'` DROP `'.$field.'`;';

					continue;
				}

				//Modify field
				$diff = array_diff_assoc($virtual_tables[$table][$field], $real_tables[$table][$field]);

				if ($diff) {
					$query[] = 'ALTER TABLE `'.$table.'` MODIFY `'.$field.'` '.$this->fieldSchema($virtual_tables[$table][$field]).';';
				}
			}
		}
		
		//Compare real_tables and virtual_tables indexes and create the query
		foreach ($virtual_indexes as $table => $fields) {
			$table_keys = array();

			foreach ($fields as $field => $key) {
				if (($key['key'] != $real_indexes[$table][$field]['key']) && ($field != 'id')) {
					$table_keys['PRIMARY']['action'] = $key['key'] ? 'ADD PRIMARY KEY' : 'DROP PRIMARY KEY';
					$table_keys['PRIMARY']['field'][] = '`'.$field.'`';
					$table_keys['PRIMARY']['name'] = '';
				}

				if ($key['unique'] != $real_indexes[$table][$field]['unique']) {
					$table_keys[$key['index']]['action'] = $key['unique'] ? 'ADD UNIQUE' : 'DROP UNIQUE';
					$table_keys[$key['index']]['field'][] = '`'.$field.'`';

					if ($key['unique']) {
						$table_keys[$key['index']]['name'] = '`'.$key['unique'].'`';
					}
				}

				if ($key['index'] != $real_indexes[$table][$field]['index']) {
					$table_keys[$key['index']]['action'] = $key['index'] ? 'ADD INDEX' : 'DROP INDEX';
					$table_keys[$key['index']]['field'][] = '`'.$field.'`';

					if ($key['index']) {
						$table_keys[$key['index']]['name'] = '`'.$key['index'].'`';
					}
				}
			}

			foreach ($table_keys as $indexes) {
				$query[] = 'ALTER TABLE `'.$table.'` '.$indexes['action'].' '.$indexes['name'].' ('.implode(',', $indexes['field']).');';
			}
		}

		return $query;
	}

	/**
	* function fieldSchema (array $field)
	*
	* Create each definition field line
	*
	* return string
	*/
	private function fieldSchema ($field) {
		$query = $field['type'];

		if ($field['length_max']) {
			$query .= '('.$field['length_max'].')';
		}

		if ($field['unsigned']) {
			$query .= ' unsigned';
		}

		$query .= $field['null'] ? ' NULL' : ' NOT NULL';

		if ($field['incremental']) {
			$query .= ' AUTO_INCREMENT PRIMARY KEY';
		} else if (isset($field['default'])) {
			$query .= ' default "'.$field['default'].'"';
		}

		return $query;
	}


	/**
	* function query (string $q)
	*
	* Launch mysql_query function
	*
	* return integer
	*/
	public function query ($q, $simulate = false) {
		$this->query = $q;
		$backtrace = debug_backtrace();
		$location = array();

		foreach ($backtrace as $k => $v) {
			$location[$k] = $v['file'].' ('.$v['line'].')';
		}

		if ($simulate) {
			$this->queryRegister[] = array(
				'status' => 'simulate',
				'query' => $q,
				'location' => $location
			);

			return true;
		} else if ($this->res = mysql_query($this->query, $this->conn)) {
			$this->queryRegister[] = array(
				'status' => 'ok',
				'query' => $q,
				'location' => $location
			);

			return $this->res;
		} else {
			global $debug;

			$this->queryRegister[] = array(
				'status' => 'ko',
				'query' => $q,
				'location' => $location
			);

			return $debug->db(mysql_error());
		}
	}

	/**
	* function loadQuery (string $q)
	*
	* Get data from a query
	*/
	public function loadQuery ($q) {
		if (!$this->query($q)) {
			return false;
		}

		$result = array();

		while ($line = mysql_fetch_assoc($this->res)) {
			$result[] = $line;
		}

		mysql_free_result($this->res);

		return $result;
	}

	/**
	* private function setFields (array $fields, string $table)
	*
	* Create the array with select fields
	*
	* return array
	*/
	private function setFields ($fields, $table) {
		foreach ($fields as $field => $newfield) {
			if (empty($table)) {
				$q_fields[] = $newfield;
			} else if (preg_match('/^[0-9]+$/', $field)) {
				$q_fields[] = $table.'.'.$newfield;
			} else if (strstr($field, '.')) {
				if (strstr($newfield, '.')) {
					$q_fields[] = $field.' AS `'.$newfield.'`';
				} else {
					$q_fields[] = $field.' AS `'.$table.'.'.$newfield.'`';
				}
			} else {
				$q_fields[] = $table.'.'.$field.' AS `'.$newfield.'`';
			}
		}

		return $q_fields;
	}

	/**
	* function select (array $select)
	*
	* Sends a SELECT db query
	*
	* return integer
	*/
	public function select ($select) {
		global $debug;

		if (!is_array($select)) {
			return array();
		}

		$q_fields = $q_tables = $q_where = $q_join = array();

		foreach ($select['fields'] as $table => $fields) {
			if (is_array($fields)) {
				$q_fields = array_merge((array)$this->setFields($fields, $table), $q_fields);
			} else {
				$q_fields = array_merge((array)$fields, $q_fields);
			}
		}

		foreach ($select['fieldsCommands'] as $command) {
			$q_fields[] = $command;
		}

		$q_tables = array_keys($select['fields']);

		if (is_array($select['join'])) {
			foreach ($select['join'] as $join) {
				$q_fields = array_merge($this->setFields($join['fields'], 'join_'.$join['table']), $q_fields);
				$q_join = array_merge(array($join['table'] => $join['conditions']), $q_join);
			}
		}

		if (!count($q_fields)) {
			return 0;
		}

		$this->query = 'SELECT '.implode(', ', $q_fields).' FROM ('.implode(', ', $q_tables).')';

		if (count($q_join)) {
			foreach ($q_join as $table => $conditions) {
				$this->query .= ' LEFT JOIN ('.$table.' as join_'.$table.') ON ('.$this->where($conditions).')';
			}
		}

		if (count($select['conditions_and'])) {
			$conditions_and = $this->where($select['conditions_and'], 'AND');
		}

		if (count($select['conditions_or'])) {
			$conditions_or = $this->where($select['conditions_or'], 'OR');
		}

		if ($conditions_and && $conditions_or) {
			$this->query .= " WHERE ($conditions_and) AND ($conditions_or)";
		} else if ($conditions_and) {
			$this->query .= ' WHERE '.$conditions_and;
		} else if ($conditions_or) {
			$this->query .= ' WHERE '.$conditions_or;
		}

		if ($select['group']) {
			if (is_array($select['group'])) {
				$this->query .= ' GROUP BY '.implode(', ', $select['group']);
			} else {
				$this->query .= ' GROUP BY '.$select['group'];
			}
		}

		if ($select['sort']) {
			if (is_array($select['sort'])) {
				$this->query .= ' ORDER BY '.implode(',', $select['sort']);
			} else {
				$this->query .= ' ORDER BY '.$select['sort'];
			}
		}

		if ($select['limit']) {
			if (is_array($select['limit'])) {
				$this->query .= ' LIMIT '.intval($select['limit'][0]).', '.intval($select['limit'][1]);
			} else {
				$this->query .= ' LIMIT '.intval($select['limit']);
			}
		}

		return $this->loadQuery($this->query.';', $store);
	}

	/**
	* public function insert (string $table, array $data)
	*
	* Execute the insert action
	*
	* return integer
	*/
	public function insert ($table, $data) {
		global $config, $debug;

		if (!$data['data'] || !is_array($data['data'])) {
			return false;
		}

		if (!is_int(key($data['data']))) {
			$data['data'] = array($data['data']);
		}

		$fields = array();

		foreach ($data['data'] as $row) {
			$fieldsRow = array_keys($row);
			$fields = array_merge($fields, $fieldsRow);
		}

		$fields = array_unique($fields);
		$query_fields = '(`'.implode('`, `', $fields).'`)';

		$values = array();

		foreach ($data['data'] as $row) {
			$valueRow = array();

			foreach ($fields as $field) {
				$valueRow[] = $row[$field];
			}

			$valueRow = '("'.implode('", "', $valueRow).'")';
			$values[] = $valueRow;
		}
		$values = implode(', ', $values);

		$query = 'INSERT INTO `'.$table.'` '.$query_fields.' VALUES '.$values.';';

		if ($this->query($query, $config->db['simulate_saves'])) {
			return $this->affected_rows();
		} else {
			return false;
		}
	}

	/**
	* public function replace (string $table, array $data)
	*
	* Execute the replace action
	*
	* return integer
	*/
	public function replace ($table, $data) {
		global $config, $debug;

		if (!$data['data'] || !is_array($data['data'])) {
			return false;
		}

		if (!is_int(key($data['data']))) {
			$data['data'] = array($data['data']);
		}

		$fields = array();

		foreach ($data['data'] as $row) {
			$fieldsRow = array_keys($row);
			$fields = array_merge($fields, $fieldsRow);
		}

		$fields = array_unique($fields);
		$query_fields = '(`'.implode('`, `', $fields).'`)';

		$values = array();

		foreach ($data['data'] as $row) {
			$valueRow = array();

			foreach ($fields as $field) {
				$valueRow[] = $row[$field];
			}

			$valueRow = '("'.implode('", "', $valueRow).'")';
			$values[] = $valueRow;
		}
		$values = implode(', ', $values);

		$query = 'REPLACE INTO `'.$table.'` '.$query_fields.' VALUES '.$values.';';

		if ($this->query($query, $config->db['simulate_saves'])) {
			return $this->affected_rows();
		} else {
			return false;
		}
	}

	/**
	* public function last_id (void)
	*
	* Return the last insert id
	*
	* return integer
	*/
	public function last_id () {
		return mysql_insert_id();
	}

	/**
	* public function affected_rows (void)
	*
	* Return the number of affected rows in the last mysql query
	*
	* return integer
	*/
	public function affected_rows () {
		return mysql_affected_rows();
	}

	/**
	* public function update (string $table, array $data)
	*
	* Execute the update action
	*
	* return integer
	*/
	public function update ($table, $data) {
		global $config, $debug;

		$query = 'UPDATE `'.$table.'` SET ';
		
		if (!$data['data'] || !is_array($data['data'])) {
			return true;
		}

		foreach ($data['data'] as $field => $value) {
			if (is_integer($field)) {
				$query .= $value.', ';
			} else {
				$query .= '`'.$field.'` = "'.$value.'", ';
			}
		}

		$query = substr($query, 0, -2);

		if (is_array($data['conditions']) && count($data['conditions']) > 0) {
			$query .= ' WHERE '.$this->where($data['conditions']);
		} else if ($data['conditions'] != 'all') {
			$debug->db(__('For security, conditions param is needed to update function. If you want update all rows, use conditions = "all"'));
			return false;
		}

		if (isset($data['limit'])) {
			$query .= ' LIMIT '.intval($data['limit']);
		}

		$query .= ';';

		if ($this->query($query, $config->db['simulate_saves'])) {
			return $this->affected_rows();
		} else {
			return false;
		}
	}

	/**
	* public function delete (string $table)
	*
	* Delete data stored in a database table
	*
	* return boolean
	*/
	public function delete ($table, $data) {
		global $debug, $config;

		$this->query = 'DELETE FROM `'.$table.'`';

		if (is_array($data['conditions']) && count($data['conditions']) > 0) {
			$this->query .= ' WHERE '.$this->where($data['conditions']);
		} else if ($data['conditions'] != 'all') {
			$debug->db(__('For security, conditions param is needed to delete function. If you want delete all rows, use conditions = "all"'));
			return false;
		}

		if ($data['limit']) {
			$this->query .= ' LIMIT '.$data['limit'];
		}

		if ($this->query($this->query, $config->db['simulate_saves'])) {
			return $this->affected_rows();
		} else {
			return false;
		}
	}

	/**
	* public function where (array $conditions)
	*
	* Create a WHERE string
	*
	* return string
	*/
	private function where ($conditions, $first = 'AND') {
		global $debug;

		$q = '';

		if ($first == 'AND') {
			$second = 'OR';
		} else {
			$first = 'OR';
			$second = 'AND';
		}

		foreach ($conditions as $key => $value) {
			if (is_int($key)) {
				if (is_array($value)) {
					$q .= ' ('.trim($this->where($value, $second)).') '.$first;
					continue;
				} else {
					$q .= ' '.$value;
				}
			} else {
				preg_match('/^([^\s]+)\s?(.*)$/', $key, $options);

				$field = trim($options[1]);
				$mode = trim($options[2]);

				if (empty($field)) {
					continue;
				}

				if (strpos($field, '.')) {
					$field = '`'.str_replace('.', '`.', $field);
				} else {
					$field = '`'.$field.'`';
				}

				$value = $this->escapeString($value);

				switch ($mode) {
					case '':
					case '=':
					case 'IN':
						if (is_array($value)) {
							$q .= ' '.$field.' IN ("'.implode('","', array_unique((array)$value)).'")';
						} else {
							$q .= ' '.$field.' = "'.$value.'"';
						}
						break;
					case '!':
						$q .= ' '.$field.' != "'.$value.'"';
						break;
					case 'BETWEEN':
						$q .= ' '.$field.' BETWEEN "'.$value[0].'" AND "'.$value[1].'"';
						break;
					case 'NOT':
						$q .= ' '.$field.' NOT IN ("'.implode('","', array_unique((array)$value)).'")';
						break;
					case '>':
					case '>=':
					case '<':
					case '<=':
					case 'LIKE':
					case 'NOT LIKE':
					case 'REGEXP':
						$q .= ' '.$field.' '.$mode.' "'.$value.'"';
						break;
				}
			}

			$q .= ' '.$first;
		}

		$q = trim($q);

		return substr($q, 0, strrpos($q, ' '));
	}

	/**
	* public function escapeString (mixed $values)
	*
	* Apply the mysql_real_escape_string to values
	*
	* return mixed
	*/
	function escapeString ($values) {
		if (is_array($values)) {
			foreach ($values as $key => $value) {
				if (is_string($value)) {
					$values[$key] = mysql_real_escape_string($value);
				} else {
					$values[$key] = $this->escapeString($value);
				}
			}

			return $values;
		} else {
			return mysql_real_escape_string($values);
		}
	}
}
?>
