<?php
/**
* phpCan v.1
*
* $Id$
*
* 2009 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Search {
	private $query = array();
	private $words = array();
	private $settings = array();

	/**
	* function like (string $table, array $fields, string $text, [array $prefs])
	*
	* Create a conditions query for "like" searches
	*/
	public function like ($settings) {
		$this->settings = $settings;

		$this->getText();

		if (!$this->settings['text'] || !$this->settings['table'] || !$this->settings['fields']) {
			return array();
		}

		if (!$this->settings['mode']) {
			$this->settings['mode'] = 'across_fields';
		}

		if (!$this->settings['min_length']) {
			$this->settings['min_length'] = 3;
		}

		global $data, $debug;

		$this->settings['fields'] = $data->autoFields($this->settings['table'], $this->settings['fields']);

		if (!$this->settings['fields']) {
			return array();
		}

		$this->likeConstruct();

		if (!$this->words) {
			return array();
		}

		$mode = 'mode_'.$this->settings['mode'];

		if (method_exists($this, $mode)) {
			$this->$mode();
		} else {
			$debug->e(__('The mode "%s" does not exists', $mode));
			return false;
		}

		if ($this->words['field']) {
			$this->fieldQuery();
		} else {
			$this->fieldSettingsQuery();
		}

		return $this->query;
	}


	/**
	* public function getQueryString (void)
	*
	* Return the last query in text format
	*/
	public function getQueryString () {
		if (!$this->words) {
			return '';
		}

		$query = '';

		foreach ((array)$this->words['field'] as $word) {
			$settings = $this->settings['field_settings'][$word['field']];
			$text = $word['text'];

			if ($settings['id_search'] && $settings['table'] && preg_match('/^[0-9]*$/', $text) && $settings['id_to_string']) {
				if ($settings['id_to_string-id_val'] && $settings['id_to_string-string_val']) {
					foreach ($settings['id_to_string'] as $v) {
						if ($v[$settings['id_to_string-id_val']] == $text) {
							$text = $v[$settings['id_to_string-string_val']];
							break;
						}
					}
				} else {
					$text = $settings['id_to_string'][$text];
				}
			}

			if (strrpos($text, ' ')) {
				$text = '"'.$text.'"';
			}

			switch ($word['type']) {
				case 'contain':
					$text = ':'.$text;
					break;
				case 'exact':
					$text = '='.$text;
					break;
				case 'start':
					$text = '^'.$text;
					break;
				case 'end':
					$text = '$'.$text;
					break;
				case 'not':
					$text = '!'.$text;
					break;
			}

			$query .= ' '.$word['field'].$text;
		}

		if ($this->words['all']) {
			$query .= ' '.implode($this->words['all'], ' ');
		}

		return trim($query);
	}


	/**
	* public function getVars (void)
	*
	* Return an array with the vars
	*/
	public function getVars () {
		if (!$this->settings['vars']) {
			return array();
		}

		$vars = array();

		foreach ((array)$this->words['field'] as $var) {
			if (in_array($var['field'], $this->settings['vars'])) {
				$vars[$var['field']] = $var['text'];
			}
		}

		return $vars;
	}


	/**
	* private function likeConstruct (void)
	*
	* Divide a text in words and search operations
	*/
	private function likeConstruct () {
		$this->words = array();

		$text = $this->settings['text'];
		$words = array();

		if (strlen($text) < $this->settings['min_length']) {
			return false;
		}

		$text = trim(preg_replace(array('/[^\w"áéíóúçñüÁÉÍÓÚÇÑÜ:=\^\$!]/', '/\s+/'), ' ', $text));

		//Search quotes
		preg_match_all('/([a-z]+(:|=|\^|\$|!))?"([^"]*)"/', $text, $quotes);

		foreach ($quotes[0] as $word) {
			$text = str_replace($word, '', $text);

			if (strlen($word) >= $this->settings['min_length']) {
				$words[] = trim($word);
			}
		}

		//Search words
		$moreWords = explode(' ', $text, 5);

		foreach ($moreWords as $word) {
			if (strlen($word) >= $this->settings['min_length']) {
				$words[] = $word;
			}
		}

		if (!$words) {
			return false;
		}

		$words = array_unique($words);

		foreach ($words as $n => $word) {
			
			//Contain
			if (strpos($word, ':')) {
				$field_word = explode(':', $word, 2);

				$this->words['field'][] = array(
					'type' => 'contain',
					'field' => $field_word[0],
					'text' => str_replace('"', '', $field_word[1])
				);

			//Exact
			} else if (strpos($word, '=')) {
				$field_word = explode('=', $word, 2);

				$this->words['field'][] = array(
					'type' => 'exact',
					'field' => $field_word[0],
					'text' => str_replace('"', '', $field_word[1])
				);

			//Start
			} else if (strpos($word, '^')) {
				$field_word = explode('^', $word, 2);

				$this->words['field'][] = array(
					'type' => 'start',
					'field' => $field_word[0],
					'text' => str_replace('"', '', $field_word[1])
				);

			//End
			} else if (strpos($word, '$')) {
				$field_word = explode('$', $word, 2);

				$this->words['field'][] = array(
					'type' => 'end',
					'field' => $field_word[0],
					'text' => str_replace('"', '', $field_word[1])
				);

			//Not
			} else if (strpos($word, '!')) {
				$field_word = explode('!', $word, 2);

				$this->words['field'][] = array(
					'type' => 'not',
					'field' => $field_word[0],
					'text' => str_replace('"', '', $field_word[1])
				);

			//All fields
			} else {
				$this->words['all'][] = str_replace('"', '', $word);
			}
		}
	}


	/**
	* private function mode_in_one_field (string $table, array $fields, array $words)
	*
	* Create conditions to search all words in one of fields
	*/
	private function mode_in_one_field ($table, $fields) {
		foreach ((array)$this->words['all'] as $w => $word) {
			foreach ($this->settings['fields'] as $f => $field) {
				$this->query['conditions_or'][$f][$w.' '.$this->settings['table'].'.'.$field.' LIKE'] = "%$word%";
			}
		}
	}


	/**
	* private function mode_across_fields (void)
	*
	* Create conditions to search all words across all fields
	*/
	private function mode_across_fields () {
		foreach ((array)$this->words['all'] as $w => $word) {
			foreach ($this->settings['fields'] as $f => $field) {
				$this->query['conditions_and'][$w][$f.' '.$this->settings['table'].'.'.$field.' LIKE'] = "%$word%";
			}
		}
	}


	/**
	* private function fieldQuery (void)
	*
	* Create conditions to search in specific fields
	*/
	private function fieldQuery () {
		global $config;

		foreach ((array)$this->words['field'] as $w => $word) {
			$field = $word['field'];
			$text = $word['text'];
			$num = $w + 100;
			
			switch ($word['type']) {
				case 'contain':
				$value = "%$text%";
				break;

				case 'exact':
				case 'not':
				$value = $text;
				break;

				case 'start':
				$value = "$text%";
				break;

				case 'end':
				$value = "%$text";
				break;

				default:
				continue 2;
			}

			if ($word['type'] == 'not') {
				$operation = ' NOT LIKE';
			} else {
				$operation = ' LIKE';
			}

			if ($settings = $this->settings['field_settings'][$field]) {
				$t = $settings['table'] ? $settings['table'] : $this->settings['table'];
				$f = $settings['field'] ? $settings['field'] : $field;
				$n = $settings['name'] ? ' ('.$settings['name'].')' : '';

				if ($settings['id_search'] && $settings['table'] && preg_match('/^[0-9]*$/', $text)) {
					$f = 'id';
					$operation = '';
					$value = $text;
				}

				if ($settings['value_function']) {
					if (function_exists($settings['value_function'])) {
						$value = $settings['value_function']($text);
					} else {
						$debug->e(__('This function does not exists (%s)', $settings['value_function']));
					}
				}

				$this->query['conditions_and'][$num.' '.$t.'.'.$f.$n.$operation] = $value;
			} else if (in_array($field, $this->settings['fields'])) {
				$this->query['conditions_and'][$num.' '.$this->settings['table'].'.'.$field.$operation] = $value;
			}
		}
	}


	/**
	* private function getText (void)
	*
	* Get the text to search
	*/
	private function getText () {
		global $vars;

		$query = '';

		if (is_string($this->settings['vars'])) {
			$this->settings['vars'] = explode(',', $this->settings['vars']);
		}

		foreach ((array)$this->settings['vars'] as $var) {
			if ($vars->var[$var]) {
				$query .= ' '.$var.':"'.$vars->str($var).'"';
			}
		}

		if ($query) {
			$this->settings['text'] = trim($query);
		}
	}
	
	
	/**
	* private function fieldSettingsQuery (void)
	*
	* Search 'all' words in related fields
	*/
	private function fieldSettingsQuery () {
		if (!$this->words['all']) {
			return false;
		}

		$sub_search = array(
			'fields' => 'id',
			'text' => implode(' ', $this->words['all'])
		);

		$ids = array();

		global $data, $debug;

		foreach ((array)$this->settings['field_settings'] as $k => $field_settings) {
			if (!$field_settings['search_in_all']) {
				continue;
			}

			$sub_search['table'] = $field_settings['table'];
			$sub_search['fields'] = $field_settings['field'];
			
			$search = new Search;

			$query = $search->like($sub_search);
			$query['fields'] = 'id';

			$result = $data->select($this->settings['table'], $query);

			foreach ($result as $value) {
				$ids[$value['id']] = true;
			}
		}

		if ($ids) {
			foreach ($this->query['conditions_and'] as $k => $v) {
				$this->query['conditions_and'][$k][$this->settings['table'].'.id'] = array_keys($ids);
			}
		}
	}


	/**
	* public function google (array $settings)
	*
	* Return array
	*/
	public function google ($settings) {
		if (!$settings['text']) {
			return array();
		}
		if (is_string($settings)) {
			$settings = array('text' => $settings);
		}

		global $config;

		$default_settings = array(
			'v' => '1.0',
			'rsz' => 'large',
			'hl' => $config->lang,
			'key' => $config->base['gsearch_key'],
			'type' => 'web',
		);

		$settings['q'] = 'site:'.$_SERVER['HTTP_HOST'].' '.trim($settings['text']);
		unset($settings['text']);

		$this->settings = array_merge($default_settings, $settings);

		$query_options = array();

		switch ($this->settings['type']) {
			case 'web':
			case 'local':
			case 'video':
			case 'blogs':
			case 'news':
			case 'books':
			case 'images':
			case 'patent':
			$url = 'http://ajax.googleapis.com/ajax/services/search/'.$this->settings['type'];
		};

		unset($this->settings['type']);

		$url = $url.http_build_query($this->settings);

		$result = $this->cUrl($url);
		$result = json_decode($result);

		return $result->responseData->results;
	}


	/**
	* private function cUrl (string $url)
	*
	* Return string
	*/
	private function cUrl ($url) {
		$curl = curl_init();

		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_REFERER, $_SERVER['HTTP_HOST']);

		$result = curl_exec($curl);

		curl_close($curl);

		return $result;
	}
}
?>