<?php
/**
* phpCan v.1
*
* $Id: class_templates.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Templates {
	public function __construct () {
		global $config;

		foreach ($config->templates as $controller => $contents) {
			if (strstr($controller, ',')) {
				$controllers = explode(',', $controller);

				foreach ($controllers as $val) {
					$config->templates[trim($val)] = array_merge((array)$config->templates[$val], (array)$contents);
				}

				unset($config->templates[$controller]);
			}
		}
	}

	/**
	* public function load (string $template)
	*
	* Load a template
	*/
	public function load ($template) {
		global $config;
		
		if ($config->templates[$template]) {
			$template = $config->templates[$template];
		} else {
			return $config->paths['includes'].'utils/empty_template.php';
		}

		global $vars;

		$dir = $vars->isAdmin() ? 'admin-templates' : 'scene-templates';
		$file = $config->paths[$dir].$template.'.php';

		if (is_file($file)) {
			return $file;
		} else {
			global $debug;

			$debug->template('The template file "'.$file.'" doesn\'t exists');

			return $config->paths['includes'].'utils/empty_template.php';
		}
	}
	
	
	/**
	* public function loadPhp (string $file)
	*
	* Load a template php file
	*/
	public function loadPhp ($file) {
		global $config, $debug, $vars;

		$dir = $vars->isAdmin() ? 'admin-templates' : 'scene-templates';
		$file = $config->paths[$dir].$file;

		if (is_file($file)) {
			return $file;
		} else {
			$debug->template('The file "'.$file.'" doesn\'t exists');

			return $config->paths['includes'].'utils/empty_template.php';
		}
	}
	
	
	/**
	* public function headLinks ()
	* 
	* Generate the code for css and js links in head
	*/
	public function headLinks () {
		global $config, $vars, $html;

		$dir = $vars->isAdmin() ? $config->paths['www-admin-templates'] : $config->paths['www-scene-templates'];
		$code = '';

		foreach ($config->templates['css'] as $value) {
			$value = trim($value);

			if (empty($value)) {
				continue;
			}

			$code .= "\n".$html->cssLink($value, $dir.'css/');
		}

		foreach ($config->templates['js'] as $value) {
			$value = trim($value);

			if (empty($value)) {
				continue;
			}

			$code .= "\n".$html->jsLink($value, $dir.'js/');
		}

		return $code;
	}


	/**
	* public function format (string $table, array $dbvalues, [string $controller], [string $suffix])
	*
	* return string
	*/
	public function format ($table, $name, $values, $def_values = array(), $controller = '', $suffix = '') {
		if (!is_array($values) || (count($values) == 0)) {
			return '';
		}

		if (!$controller) {
			global $vars;
			$controller = $vars->getController();
		}

		global $config, $debug, $data;

		ob_start();

		$object = $config->tables[$table][$name];

		$template = $config->paths['admin-templates'].'/formats/'.$object->format.'/'.$object->format.'-'.$controller.'.php';

		if (!is_file($template)) {
			$template = $config->paths['admin-templates'].'/formats/default/default-'.$controller.'.php';
		}

		if (!is_file($template)) {
			$debug->template($settings['format']);

			return '';
		}

		$info = array(
			'name' => $name,
			'suffix' => $suffix,
			'value' => $values,
			'default' => (array)$def_values,
			'error' => $data->error[$table][$name],
			'object' => $object
		);

		if (is_array($info['value']) && (count($info['value']) == 1)) {
			$info['value'] = current($info['value']);
			$info['default'] = current($info['default']);
		}

		if (is_array($info['error']) && (count($info['error']) == 1)) {
			$info['error'] = current($info['error']);
		}

		include ($template);

		$txt = ob_get_contents();

		ob_end_clean();

		return $txt;
	}


	/**
	* public function assignTemplate (string $name, string $template)
	*
	* Assing a template to a $name posicion
	*
	*/
	public function assingTemplate ($name, $template) {
		global $config, $vars;

		$config->templates[$vars->getController()][$name] = $template;
	}
}
?>
