<?php
/**
* phpCan v.1
*
* $Id: class_thumb.php 194 2009-06-25 16:19:56Z eordes $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Thumb {
	private $wrapper = array(
		'width' => 'w',
		'height' => 'h',
		'format' => 'f',
		'allowEnlargement' => 'aoe',
		'zoomCrop' => 'zc',
		'keepProportions' => 'iar',
		'brightness' => 'brit',
		'contrast' => 'cont',
		'gamma' => 'gam',
		'saturation' => 'sat',
		'grayscale' => 'gray',
		'threshold' => 'th',
		'reduceColors' => 'rdc',
		'colorize' => 'clr',
		'unsharp' => 'usm',
		'blur' => 'blur',
		'smooth' => 'smth',
		'levels' => 'lvl',
		'overlay' => 'over',
		'watermarkImage' => 'wmi',
		'watermarkText' => 'wmt',
		'flip' => 'flip',
		'roundCorners' => 'ric',
		'applyMask' => 'mask',
		'dropShadow' => 'drop',
		'crop' => 'crop',
		'nocache' => 'nocache',
		'rotate' => 'rot',
		'transparentColor' => 'stc'
	);
	private $filters = array(
		'reduceColors' => array('number', 'dithering'),
		'colorize' => array('amount', 'color'),
		'unsharp' => array('amount', 'radius', 'threshold'),
		'levels' => array('channel', 'method', 'threshold'),
		'overlay' => array('image', 'order', 'margin', 'opacity'),
		'watermarkImage' => array('fileName', 'alignment', 'opacity', 'x', 'y', 'rotation'),
		'watermarkText' => array('text', 'fontSize', 'align', 'textColor', 'fontFileName', 'opacity', 'edge', 'angle', 'backgroundColor', 'backgroundOpacity'),
		'flip' => array('direction'),
		'roundCorners' => array('xRadius', 'yRadius'),
		'grayscale' => array('desaturate'),
		'applyMask' => array('fileName'),
		'dropShadow' => array('distance', 'fadeWidth', 'color', 'angle'),
		'crop' => array('left', 'right', 'top', 'bottom'),
		'rotate' => array('angle', 'backgroundColor'),
		'transparentColor' => array('color', 'minThreshold', 'maxThreshold')
	);
	private $command;
	private $process;
	private $image;


	/**
	* public function createCommand ([array $process])
	*
	* Return string
	*/
	private function createCommand ($process = array()) {
		$command = '';

		foreach ($process as $key => $value) {
			if (!isset($this->wrapper[$key])) {
				continue;
			}

			if (isset($this->filters[$key])) {
				$filterData = '';
				foreach ($this->filters[$key] as $indexkey) {
					$filterData .= '|'.$value[$indexkey];
				}
				$command .= '&amp;fltr[]='.$this->wrapper[$key].$filterData;
			} else {
				$command .= '&amp;'.$this->wrapper[$key].'='.$value;
			}
		}

		return $command;
	}


	/**
	* public function mergeOptions ([int $width], [int $height], [array $process])
	*
	* Return array
	*/
	private function mergeOptions ($width = 0, $height = 0, $process = array()) {
		if (!is_array($width)) {
			$width = $width ? array('width' => intval($width)) : array();
		}

		if (!is_array($height)) {
			$height = $height ? array('height' => intval($height)) : array();
		}

		$process = array_merge((array)$process, $width, $height);

		return $process;
	}


	/**
	* public function config ([int $width], [int $height], [array $process])
	*
	* Return none
	*/
	public function config ($width = 0, $height = 0, $process = array()) {
		$this->process = $this->mergeOptions($width, $height, $process);
		$this->command = $this->createCommand($this->process);
	}


	/**
	* public function src (string $image, [int $width], [int $height], [array $process])
	*
	* Return string
	*/
	public function src ($image, $width = 0, $height = 0, $process = array()) {
		if (is_array($image)) {
			$width = $image;
			$image = $image['src'];
			unset($width['src']);
		}

		if (empty($image)) {
			return false;
		} else {
			$this->image = $image;
		}

		$this->config($width, $height, $process);

		global $config;
		return $config->paths['www'].'img.php?src='.$this->image.$this->command;
	}


	/**
	* public function img (string $image, [int $width], [int $height], [array $process])
	*
	* Return string
	*/
	public function img ($image, $width = 0, $height = 0, $process = array()) {
		global $html;

		return $html->img($this->src($image, $width, $height, $process));
	}


	/**
	* public function live (string $image, [int $width], [int $height], [array $process])
	*
	* Return string
	*/
	public function liveSrc ($image, $width = 0, $height = 0, $process = array()) {
		$process['nocache'] = true;

		return $this->src($image, $width, $height, $process);
	}


	/**
	* public function liveImg (string $image, [int $width], [int $height], [array $process])
	*
	* Return string
	*/
	public function liveImg ($image, $width = 0, $height = 0, $process = array()) {
		global $html;

		return $html->img($this->liveSrc($image, $width, $height, $process));
	}


	/**
	* public function duplicateSrc ([string $image], [array $options])
	*
	* Return string
	*/
	public function duplicateSrc ($image = '', $width = 0, $height = 0, $process = array()) {
		if (is_array($image)) {
			$width = $image;
			$image = $image['src'];
			unset($width['src']);
		}

		if (!$image) {
			$image = $this->image;
		}

		if ($process = $this->mergeOptions($width, $height, $process)) {
			$process = array_merge($this->process, $process);
			$command = $this->createCommand($process);
		} else {
			$command = $this->command;
		}

		global $config;
		return $config->paths['www'].'img.php?src='.$image.$command;
	}


	/**
	* public function duplicateImg ([string $image], [array $options])
	*
	* Return string
	*/
	public function duplicateImg ($image = '', $width = 0, $height = 0, $process = array()) {
		global $html;

		return $html->img($this->duplicateSrc($image, $width, $height, $process));
	}

}
?>
