<?php
/**
* phpCan v.1
*
* $Id: class_vars.php 339 2009-07-17 16:12:33Z lito $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Vars {
	public $var;
	public $rewrite;
	public $table;
	public $message = array();
	public $overload = false;

	private $controller;
	private $scene;
	private $admin = false;

	/**
	* function load (void)
	*
	* Load the default config
	*/
	public function load () {
		global $config, $debug;

		//Detect redirect from ajax
		if ($_GET['phpcan_ajax'] == 1) {
			unset($_GET['phpcan_ajax']);

			$_SERVER['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest';
		}

		//Detect post content length
		if (isset($_SERVER['CONTENT_LENGTH'])) {
			if ($_SERVER['CONTENT_LENGTH'] > toBytes(ini_get('upload_max_filesize'))) {
				$this->overload = true;
			}
		}

		//Get path
		$url = preg_replace('|^'.$config->paths['www'].'|', '', getenv('REQUEST_URI'));
		$url = parse_url($url);

		$tmp_path = explode('/', $url['path']);

		//Remove empty vars
		$path = array();

		foreach ($tmp_path as $value) {
			if (!empty($value)) {
				$path[] = $value;
			}
		}

		//Get scene
		if ($config->scenes[$path[0]]) {
			$this->scene = array_shift($path);
		} else if ($config->scenes['main']) {
			$this->scene = 'main';
		} else {
			$debug->vars('There is not "main" or "'.$path[0].'" scene');
		}

		//Get admin
		if ($path[0] == 'admin') {
			$this->admin = true;
			array_shift($path);
		}

		$this->var = $this->rewrite = $path;
		$this->var['anchor'] = $url['fragment'];

		//Save the rest of varibles in $this->var
		$this->var += ($_GET + $_POST + $_FILES + $_COOKIE);

		//Save message
		$this->message['inbox'] = $this->var['phpcan_message'];
		$this->message['type'] = $this->var['phpcan_message_type'];
		$this->message['outbox'] = '';

		deleteCookie('phpcan_message');
		deleteCookie('phpcan_message_type');
	}

	/**
	* function setController (string $controller)
	*
	* Assigns a new controller
	*/ 
	public function setController ($controller = null) {
		global $config, $debug;

		$remove_first_var = false;

		if (is_null($controller)) {
			$controller = $this->var[0];
			$remove_first_var = true;
		}

		$controller = preg_replace('/\W/', '-', trim($controller));
		$controller = strtolower($controller);

		if (!$controller) {
			$this->controller = 'index';
		} else if (isset($config->data[$controller]) || isset($config->templates[$controller])) {
			$this->controller = $controller;

			if ($remove_first_var) {
				array_shift($this->var);
				array_shift($this->rewrite);
			}
		} else {
			$this->controller = 'undefined';
		}
	}

	/**
	* function getController (string [$controller])
	*
	* returns the $this->controller value
	*
	* return string
	*/
	public function getController ($controller = '') {
		if (!$controller) {
			return $this->controller;
		} else {
			return ($this->controller == $controller) ? true : false;
		}
	}

	/**
	* function setScene (void)
	*
	* assing the $this->scene value
	*/
	public function setScene ($scene) {
		$this->scene =  $scene;
	}

	/**
	* function getScene (void)
	*
	* returns the $this->scene value
	*
	* return string
	*/
	public function getScene ($scene = '') {
		if (!$scene) {
			return $this->scene;
		} else {
			return ($this->scene ==  $scene) ? true : false;
		}
	}

	/**
	* function isAdmin (void)
	*
	* returns the $this->admin value
	*
	* return boolean
	*/
	public function isAdmin () {
		return $this->admin;
	}

	/**
	* function getTableVars (string $table, [string $suffix])
	*
	* returns the vars of a table
	*
	* return array
	*/
	public function getTableVars ($table, $suffix = '') {
		global $config, $debug;

		if (!$config->fields[$table]) {
			return false;
		}

		$vars = array();

		//Table fields
		foreach ($config->fields[$table] as $field => $v) {
			if ($field == 'id') {
				continue;
			}

			if (isset($this->var[$field.$suffix])) {
				$vars[$field] = $this->var[$field.$suffix];
			}
		}

		//Relations fields
		if (is_array($config->relations[$table])) {
			foreach ($config->relations[$table] as $table_relation) {
				foreach ($table_relation as $relation) {
					if ($relation['mode'] == 'x 1' && array_key_exists($relation['join'].$suffix, $this->var)) {
						$vars[$relation['join']] = $this->var[$relation['join'].$suffix];
					}
				}
			}
		}

		return $vars;
	}

	/**
	* function deleteTableVars (string $table, [string $suffix])
	*
	*/
	public function deleteTableVars ($table, $suffix = '') {
		global $config;

		if (!$config->fields[$table]) {
			return false;
		}

		foreach ($config->fields[$table] as $field => $v) {
			unset ($this->var[$field.$suffix]);
		}
	}

	/**
	* function delete (string $name)
	*
	*/
	public function delete ($name) {
		if ($name === 0 || $name === '0') {
			array_shift($this->var);
			array_shift($this->rewrite);
		} else {
			unset ($this->var[$name], $_GET[$name], $_POST[$name], $_FILES[$name], $_COOKIE[$name]);
		}
	}

	/**
	* function message (string $message, string $type)
	*
	* save/return a message
	*
	* return string
	*/
	public function message ($message = '', $type = '') {
		if (empty($message)) {
			return $this->message['inbox'];
		} else {
			$this->message['inbox'] = $message;
			$this->message['outbox'] = $message;
			$this->message['type'] = $type;
		}
	}
	
	/**
	* function messageExists (void)
	*
	* return boolean
	*/
	public function messageExists () {
		return $this->message['inbox'] ? true : false;
	}

	/**
	* function messageType (string $type)
	*
	* return boolean or string
	*/
	public function messageType ($type = '') {
		if (empty($type)) {
			return $this->message['type'];
		} else {
			return ($this->message['type'] == $type) ? true : false;
		}
	}

	/**
	* function getKeyVar (string $name, [string $key], [string $separation])
	*
	* return boolean or string
	*/
	public function getKeyVar ($name, $key = false, $separation = '-') {
		if (!$this->var[$name]) {
			return false;
		}

		$key_value = explode($separation, $this->var[$name], 2);

		if ($key) {
			return $key_value[0] == $key ? true : false;
		} else {
			return $key_value[0];
		}
	}

	/**
	* function getValueVar (string $name, [string $value], [string $separation])
	*
	* return boolean or string
	*/
	public function getValueVar ($name, $value = false, $separation = '-') {
		if (!$this->var[$name]) {
			return false;
		}

		$key_value = explode($separation, $this->var[$name], 2);

		if ($value) {
			return $key_value[1] == $value ? true : false;
		} else {
			return $key_value[1];
		}
	}

	/**
	* function int (string $name)
	*
	* return integer
	*/
	public function int ($name) {
		if (!$this->var[$name]) {
			return 0;
		} else {
			return intval($this->var[$name]);
		}
	}
	
	/**
	* function str (string $name)
	*
	* return string
	*/
	public function str ($name) {
		$string = urldecode(trim($this->var[$name]));

		if(get_magic_quotes_gpc()) {
			return stripslashes($string);
		} else {
			return $string;
		}
	}

	/**
	* function arr (string $name)
	*
	* return array
	*/
	public function arr ($name) {
		if (isset($this->var[$name])) {
			if (is_array($this->var[$name])) {
				return $this->var[$name];
			}

			return array($this->var[$name]);
		}

		return array();
	}
}
?>
