<?php
/**
* phpCan v.1
*
* $Id: functions.php 350 2009-07-23 20:01:12Z lito $
*
* 2008 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

/**
* function filename (string $file)
*
* Returns a correct filename
*
*	return string
*/
function filename ($file) {
	return preg_replace(array('/\.+/', '/[^\w\.-]/'), array('.', '_'), $file);
}

/**
* function strip_magic_slashes (string $str)
*
* Returns the correct value undoing magic quotes
*
* return string
*/
function strip_magic_slashes ($str) {
	return get_magic_quotes_gpc()?stripslashes($str):$str;
}

/**
* function isAjax (void)
*
* Detects an Ajax load
*
* return boolean
*/
function isAjax () {
	global $vars;

	return ($vars->var['phpcan_ajax'] || ($_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest'));
}

/**
* function encrypt (string $text)
*
* Text encryption
*
* return string
*/
function encrypt ($text) {
	global $config;

	return trim(base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $config->base['key'], $text, MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND))));
}

/**
* function decrypt (string $text)
*
* Text decryption
*
* return string
*/
function decrypt ($text) {
	global $config;

	return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $config->base['key'], base64_decode($text), MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)));
}

/**
* function recursiveFolder (string $folder)
*
* Create a folder recursively
*
* return boolean
*/
function recursiveFolder ($folder) {
	if (is_dir($folder)) {
		return true;
	}

	$dirs = explode('/', $folder);
	$dir = array_shift($dirs);

	if (!strpos($dir, ':')) {
		$dir = '/'.$dir;
	}

	foreach ($dirs as $part) {
		if (empty($part) || ($part == '.')) {
			continue;
		}

		$dir .= '/'.$part;

		if ($part == '..') {
			continue;
		}

		if (!is_dir($dir)) {
			$ok = @mkdir($dir, 0755);

			if (!$ok) {
				return false;
			}
		}
	}

	clearstatcache();

	return is_dir($folder);
}

/**
* function __autoload($class_name)
*/
function __autoload ($class_name) {
	$file = 'class_'.strtolower($class_name).'.php';

	if (is_file(INCLUDES_PATH.$file)) {
		include_once (INCLUDES_PATH.$file);
	} else if (is_file(INCLUDES_PATH.$class_name.$file)) {
		include_once (INCLUDES_PATH.$class_name.$file);
	}
}

/**
* function __($text)
*
* return string
*/
function __($text, $args = null) {
	global $gettext_tables;

	$text = is_object($gettext_tables)?$gettext_tables->translate($text):$text;

	if (is_null($args)) {
		return $text;
	} else if (is_array($args)) {
		return vsprintf($text, $args);
	} else {
		$args = func_get_args();

		array_shift($args);

		$vsprintf_args = array();

		foreach ($args as $arg) {
			$vsprintf_args[] = $arg;
		}

		return vsprintf($text, $vsprintf_args);
	}
}

/**
* function __e($text)
*
* echo string
*/
function __e($text, $args = null) {
	if (count(func_get_args()) > 2) {
		$args = func_get_args();

		array_shift($args);
	}

	echo __($text, $args);
}

/**
* function array2json(array $array)
*
* return string
*/
function array2json($array) {
	if (function_exists('json_encode')) {
		return json_encode($array);
	}

	$parts = array();
	$is_list = false; //Default: asociative array

	//Is numerical?
	$keys = array_keys($array);
	$max_length = count($array)-1;

    if (($keys[0] == 0) and ($keys[$max_length] == $max_length)) {
    	$is_list = true;

		for ($i=0; $i<count($keys); $i++) {
			if($i != $keys[$i]) {
				$is_list = false;
				break;
			}
		}
	}

	foreach ($array as $key => $value) {
		if (is_array($value)) {
			if ($is_list) {
				$parts[] = array2json($value); //Recursive
			} else {
				$parts[] = '"' . $key . '":' . array2json($value); //Recursive
			}

		} else {
			$str = '';
			if (!$is_list) {
				$str = '"' . $key . '":';
			}

			if (is_numeric($value)) {
				$str .= $value;
			} else if ($value === false) {
				$str .= 'false';
			} else if ($value === true) {
				$str .= 'true';
			} else {
				$str .= '"' . addslashes($value) . '"';
			}

			$parts[] = $str;
		}
	}
	$json = implode(',',$parts);

	if ($is_list) {
		return '[' . $json . ']';
	}

    return '{' . $json . '}';
}


/**
* function saveCookie(string $name, string $value, int $duration)
*
* Save a cookie in the actual scene
*/
function saveCookie($name, $value, $duration) {
	global $config, $vars;

	$vars->var[$name] = $value;

	$path = $vars->isAdmin() ? $config->paths['www-admin'] : $config->paths['www-scene'];

	setcookie($name, $value, time() + $duration, $path);
}


/**
* function deleteCookie(string $name)
*
* Delete a cookie in the actual scene
*/
function deleteCookie($name) {
	global $config, $vars;

	unset($vars->var[$name]);

	$path = $config->paths['www'];

	if ($vars->isAdmin()) {
		$path .= 'admin/';
	}

	setcookie($name, '', time() - 60, $path);
}


/**
* function path ([string], [string], [...])
*
* Return string
*/
function path () {
	global $vars, $config;

	$args = func_get_args();

	if ($vars->isAdmin()) {
		$path = $config->paths['www-admin'];
	} else {
		$path = $config->paths['www-scene'];
	}

	if ($config->base['language_detect'] == 'subfolder') {
		if (strpos($args[0], 'lang:') === 0) {
			$path .= substr($args[0], 5).'/';
			array_shift($args);
		} else {
			$path .= $config->lang.'/';
		}
	}

	if (!$args) {
		if ($vars->getController() && !$vars->getController('index')) {
			$path .= $vars->getController().'/';
		}

		if ($vars->rewrite) {
			$path .= implode('/', $vars->rewrite).'/';
		}

		return $path;
	} else {
		if ($args[0] === true) {
			$path .= $vars->getController().'/';
		} else if (!empty($args[0])) {
			$path .= $args[0].'/';
		}

		unset($args[0]);

		foreach ((array)$args as $key => $value) {
			if (($value === true) && $vars->rewrite[$key-1]) {
				$path .= $vars->rewrite[$key-1].'/';
			} else if (!empty($value)) {
				$path .= $value.'/';
			}
		}

		return $path;
	}
}


/**
* function arrayGet ([string $name], [string $value], [bool $merge_get])
* function arrayGet ([array $values], [bool $merge_get])
*
* Return string
*/
function arrayGet ($name = null, $value = null, $merge_get = true) {
	global $vars, $debug;

	if (is_array($name)) {
		$values = $name;
		$merge_get = $value;
	} else if (is_string($name) && !is_array($value)) {
		$values = array($name => $value);
	} else {
		$values = array();
		$merge_get = is_null($value) ? true : $value;
	}

	if ($merge_get === true) {
		$get = array_merge($_GET, $values);
	} else if (is_array($merge_get)) {
		$get = array();

		foreach ($merge_get as $value) {
			$get[$value] = $_GET[$value];
		}
		$get = array_merge($get, $values);
	} else {
		$get = $values;
	}

	foreach ($get as $name => $value) {
		if (is_null($value)) {
			unset($get[$name]);
		} else {
			$get[$name] = strip_magic_slashes($value);
		}
	}

	return $get;
}


/**
* function get ([string $name], [string $value], [bool $merge_get])
* function get ([array $values], [bool $merge_get])
*
* Return string
*/
function get ($name = null, $value = null, $merge_get = true) {
	$get = arrayGet($name, $value, $merge_get);
	$get = http_build_query($get);

	return $get ? '?'.$get : '';
}


/**
* function redirect (string $url, boolean $ajax)
*
* URI redirect
*/
function redirect ($url, $ajax = true) {
	global $vars;

	if ($vars->message['outbox']) {
		saveCookie('phpcan_message', $vars->message['outbox'], 120);
		saveCookie('phpcan_message_type', $vars->message['type'], 120);
	}

	if ($ajax && isAjax()) {
		$url .= (strrchr($url, '?')) ? '&phpcan_ajax=1' : '?phpcan_ajax=1';
	}

	header('Location: '.$url);
	exit;
}

/**
* function toBytes (string $size)
*
* return integer
*/
function toBytes ($size) {
	$size = trim(strtolower($size));
	$last = substr($size, -1);
	$size = intval($size);

	switch ($last) {
		case 'g':
			$size *= 1024;
		case 'm':
			$size *= 1024;
		case 'k':
			$size *= 1024;
	}

	return $size;
}

/**
* function relativeTime (string $date, [int $now])
*
* Return string
*/
function relativeTime ($date, $now = 0) {
	if (!$date) {
		return false;
	} else if (!is_int($date)) {
		$date = strtotime($date);
	}

	if (!$now) {
		$now = time();
	} else if (!is_int($now)) {
		$now = strtotime($now);
	}

	$diff = $now - $date;

	//Future
	if ($diff < 0) {
		return false;
	}
	//Less than 1 minute
	if ($diff < 60) {
		return 'hai uns segundos';
	}
	//Less than 1 hour
	if ($diff < 3600) {
		return 'hai uns minutos';
	}
	//Less than 1 day
	if ($diff < 86400) {
		return 'hai '.round($diff/3600).' horas';
	}
	//Less than 1 week
	if ($diff < 604800) {
		return 'hai '.round($diff/86400).' días';
	}
	//Less than 1 mounth
	if ($diff < 2419200) {
		return 'hai '.round($diff/604800).' semanas';
	}
	//Less than 1 year
	if ($diff < 31536000) {
		return 'hai '.round($diff/2592000).' meses';
	}

	return date('F j, Y', $date);
}

/**
* function escapeForm (string $data)
*
* Return string
*/
function escapeForm ($data) {
	$in = array('&','\\"','"','<','>','&amp;amp;');
	$out = array('&amp;','&quot;','&quot;','&lt;','&gt;','&amp;');

	if (is_array($data)) {
		foreach ($data as $key => $value) {
			if (is_array($value)) {
				$data[$key] = escapeForm($value);
			} else {
				$data[$key] = str_replace($in, $out, $value);
			}
		}
	} else {
		$data = str_replace($in, $out, $data);
	}

	return $data;
}
?>
